package cn.iocoder.yudao.module.basic.controller.admin.inputsinfo;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.*;
import cn.iocoder.yudao.module.basic.dal.dataobject.inputsinfo.InputsInfoDO;
import cn.iocoder.yudao.module.basic.convert.inputsinfo.InputsInfoConvert;
import cn.iocoder.yudao.module.basic.service.inputsinfo.InputsInfoService;

@Api(tags = "管理后台 - 投入品")
@RestController
@RequestMapping("/basic/inputs-info")
@Validated
public class InputsInfoController {

    @Resource
    private InputsInfoService inputsInfoService;

    @PostMapping("/create")
    @ApiOperation("创建投入品")
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:create')")
    public CommonResult<Long> createInputsInfo(@Valid @RequestBody InputsInfoCreateReqVO createReqVO) {
        return success(inputsInfoService.createInputsInfo(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新投入品")
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:update')")
    public CommonResult<Boolean> updateInputsInfo(@Valid @RequestBody InputsInfoUpdateReqVO updateReqVO) {
        inputsInfoService.updateInputsInfo(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除投入品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:delete')")
    public CommonResult<Boolean> deleteInputsInfo(@RequestParam("id") Long id) {
        inputsInfoService.deleteInputsInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得投入品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:query')")
    public CommonResult<InputsInfoRespVO> getInputsInfo(@RequestParam("id") Long id) {
        InputsInfoDO inputsInfo = inputsInfoService.getInputsInfo(id);
        return success(InputsInfoConvert.INSTANCE.convert(inputsInfo));
    }

    @GetMapping("/page")
    @ApiOperation("获得投入品分页")
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:query')")
    public CommonResult<PageResult<InputsInfoRespVO>> getInputsInfoPage(@Valid InputsInfoPageReqVO pageVO) {
        PageResult<InputsInfoDO> pageResult = inputsInfoService.getInputsInfoPage(pageVO);
        return success(InputsInfoConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/queryListOfMiniResult")
    @ApiOperation("获得投入品信息（返回字段做精简化）")
    @ApiImplicitParam(name = "businessInfoId", value = "商户信息ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:query')")
    public CommonResult<List<InputsInfoForTransferRespVO>> queryListOfMiniResult(@RequestParam("businessInfoId") Long businessInfoId) {
        return success(inputsInfoService.queryListOfMiniResult(businessInfoId));
    }

    @GetMapping("/getInputsInfoInputsInfoList")
    @ApiOperation("获得所有投入品信息")
    @PreAuthorize("@ss.hasPermission('basic:inputs-info:query')")
    public CommonResult<List<InputsInfoDO>> getInputsInfoInputsInfoList() {
        List<InputsInfoDO> inputsInfoInputsInfoList = inputsInfoService.getInputsInfoInputsInfoList();
        return success(inputsInfoInputsInfoList);
    }

    @GetMapping("/getInputsInfoListByBelongBusinessId")
    @ApiOperation("根据指定的商户id 获取所有名下多个投入品")
    @ApiImplicitParam(name = "belongBusinessId", value = "商户ID", required = true, example = "1", dataTypeClass = List.class)
    public CommonResult<List<InputsInfoDO>> getInputsInfoListByBelongBusinessId(@RequestParam("belongBusinessId") List<Long> belongBusinessId) {
        List<InputsInfoDO> inputsInfoListByBelongBusinessId = inputsInfoService.getInputsInfoListByBelongBusinessId(belongBusinessId);
        return success(inputsInfoListByBelongBusinessId);
    }
}
