package cn.iocoder.yudao.module.basic.service.goodsinputrelational;

import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.basic.api.goodsinfo.dto.GoodsInfoRespDTO;
import cn.iocoder.yudao.module.basic.api.goodsinputrelational.dto.InputInfoRelationalRespDTO;
import cn.iocoder.yudao.module.basic.api.goodsinputrelational.dto.RelationalRespDTO;
import cn.iocoder.yudao.module.basic.controller.admin.goodsinputrelational.vo.GoodsInputRelationalCreateBatchReqVO;
import cn.iocoder.yudao.module.basic.controller.admin.goodsinputrelational.vo.GoodsInputRelationalCreateReqVO;
import cn.iocoder.yudao.module.basic.convert.goodsinfo.GoodsInfoConvert;
import cn.iocoder.yudao.module.basic.convert.inputsinfo.InputsInfoConvert;
import cn.iocoder.yudao.module.basic.dal.dataobject.goodsinfo.GoodsInfoDO;
import cn.iocoder.yudao.module.basic.dal.dataobject.goodsinputrelational.GoodsInputRelationalDO;
import cn.iocoder.yudao.module.basic.dal.dataobject.inputsinfo.InputsInfoDO;
import cn.iocoder.yudao.module.basic.dal.mysql.goodsinfo.GoodsInfoMapper;
import cn.iocoder.yudao.module.basic.dal.mysql.goodsinputrelational.GoodsInputRelationalMapper;
import cn.iocoder.yudao.module.basic.dal.mysql.inputsinfo.InputsInfoMapper;
import cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.basic.service.inputsinfo.InputsInfoService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;

/**
 * 投入品与商品映射 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class GoodsInputRelationalServiceImpl implements GoodsInputRelationalService {

    @Resource
    private GoodsInputRelationalMapper goodsInputRelationalMapper;

    @Resource
    private GoodsInfoMapper goodsInfoMapper;

    @Resource
    private InputsInfoMapper inputsInfoMapper;

    @Resource
    private InputsInfoService inputsInfoService;

    /**
     * 根据商品编码集合查询所有投入品信息
     * @param goodsIds
     * @return
     */
    @Override
    public RelationalRespDTO getInputInfoByGoodsId(List<String> goodsIds) {
        List<GoodsInfoDO> goodsInfos = goodsInfoMapper.selectList(new LambdaQueryWrapperX<GoodsInfoDO>().in(GoodsInfoDO::getGoodsId, goodsIds));
        if(goodsInfos.size() != goodsIds.size()) {
            throw exception(ErrorCodeConstants.GOODS_INFO_NOT_EXISTS);
        }

        List<GoodsInputRelationalDO> relationals = goodsInputRelationalMapper.selectList(
                new LambdaQueryWrapperX<GoodsInputRelationalDO>().in(GoodsInputRelationalDO::getGoodsId, goodsIds));
        Map<String, String> relaIds = new HashMap<>();// 关联表键值对集合 <投入品编码, 商品编码>
        List<String> inRelaGoodsIds = new ArrayList<>();// 有在关联表里面的商品编码
        List<String> inputsIds = new ArrayList<>();
        for (GoodsInputRelationalDO rela: relationals) {
            relaIds.put(rela.getInputsId(), rela.getGoodsId());
            inputsIds.add(rela.getInputsId());
            inRelaGoodsIds.add(rela.getGoodsId());
        }
        RelationalRespDTO result = new RelationalRespDTO();
        if(relationals.size() != goodsIds.size()) {
            // 返回不在关联表里面的商品
            List<String> notInRelaGoodsIds = goodsIds.stream().filter(ids -> !inRelaGoodsIds.contains(ids)).collect(Collectors.toList());
            List<GoodsInfoDO> goodsInfoDOs = goodsInfoMapper.selectList(new LambdaQueryWrapperX<GoodsInfoDO>().in(GoodsInfoDO::getGoodsId, notInRelaGoodsIds));
            List<GoodsInfoRespDTO> goodsInfoRespDTO = GoodsInfoConvert.INSTANCE.convertToDTOList(goodsInfoDOs);
            result.setGoodsInfos(goodsInfoRespDTO);
            return result;
        }

        // 查询有关联的投入品
        List<InputsInfoDO> inputsInfos = inputsInfoMapper.selectList(
                new LambdaQueryWrapperX<InputsInfoDO>().in(InputsInfoDO::getInputsId, inputsIds));
        List<InputInfoRelationalRespDTO> inputsInfoRespDTO = InputsInfoConvert.INSTANCE.convertRelationalRespList(inputsInfos);
        for (InputInfoRelationalRespDTO respDTO: inputsInfoRespDTO) {
            respDTO.setGoodsId(relaIds.get(respDTO.getInputsId()));
            // respDTO.setGoodsName()
        }
        result.setInputsInfo(inputsInfoRespDTO);
        return result;
    }

    @Override
    @Transactional
    public void createGoodsInputRelationalBatch(GoodsInputRelationalCreateBatchReqVO createReqVO) {
        List<String> goodsIds = new ArrayList<>();
        // 需要绑定到已有投入品的商品集合
        List<String> existInputsIds = new ArrayList<>();
        Map<String, GoodsInfoDO> goodsInfoMap = new HashMap<>();
        for(GoodsInputRelationalCreateReqVO req: createReqVO.getGoodsInputRelationals()) {
            goodsIds.add(req.getGoodsId());
            if(req.getInputsId() != null) {
                existInputsIds.add(req.getInputsId());
            }
        }

        // 判断是否已绑定过
        List<GoodsInputRelationalDO> goodsInputRelationalDO = goodsInputRelationalMapper.selectList(
                new LambdaQueryWrapperX<GoodsInputRelationalDO>().in(GoodsInputRelationalDO::getGoodsId, goodsIds));
        if(!goodsInputRelationalDO.isEmpty()) {
            throw exception(ErrorCodeConstants.EXIST_GOODS_HAS_BIND_INPUT);
        }

        // 查询商品信息集合
        List<GoodsInfoDO> goodsInfoDOs = goodsInfoMapper.selectList(new LambdaQueryWrapperX<GoodsInfoDO>().in(GoodsInfoDO::getGoodsId, goodsIds));
        if(goodsInfoDOs.size() != goodsIds.size()) {
            throw exception(ErrorCodeConstants.GOODS_INFO_NOT_EXISTS);
        }
        for(GoodsInfoDO goods: goodsInfoDOs) {
            goodsInfoMap.put(goods.getGoodsId(), goods);
        }

        // 查询投入品信息集合
        if(!existInputsIds.isEmpty()) {
            List<InputsInfoDO> existInputs = inputsInfoMapper.selectList(new LambdaQueryWrapperX<InputsInfoDO>().in(InputsInfoDO::getInputsId, existInputsIds));
            if(existInputsIds.size() != existInputs.size()) { // 验证投入品
                throw exception(ErrorCodeConstants.INPUTS_INFO_NOT_EXISTS);
            }
        }

        List<InputsInfoDO> inputsInfoDOs = new ArrayList<>();
        List<GoodsInputRelationalDO> goodsInputRelationalDOs = new ArrayList<>();
        for(GoodsInputRelationalCreateReqVO item: createReqVO.getGoodsInputRelationals()) {
            GoodsInputRelationalDO relationalDO = new GoodsInputRelationalDO();
            relationalDO.setGoodsId(item.getGoodsId());

            // 判断是否传了投入品id
            if (item.getInputsId() == null) {
                // 新增投入品流程
                GoodsInfoDO tempGoods = goodsInfoMap.get(item.getGoodsId());
                InputsInfoDO inputsInfoDO = new InputsInfoDO();
                inputsInfoDO.setInputsName(tempGoods.getGoodsName());
                inputsInfoDO.setIsBaseUnit(tempGoods.getIsBaseUnit());
                inputsInfoDO.setInputsUnitId(tempGoods.getGoodsUnitId());
                inputsInfoDO.setInputsSpecs(tempGoods.getGoodsSpecs());
                inputsInfoDO.setConversionValue(tempGoods.getConversionValue());
                inputsInfoDO.setInputsId(CodeEnum.INPUTS_INFO_CODE.getCode());
                inputsInfoDO.setInputsRemarks(tempGoods.getGoodsRemarks());
                inputsInfoDO.setBelongBusinessId(tempGoods.getBelongBusinessId());
                inputsInfoDO.setBelongBusinessName(tempGoods.getBelongBusinessName());
                inputsInfoDO.setInputsPhotoUrl(tempGoods.getGoodsImgUrl());
                inputsInfoDO.setShelfLife(tempGoods.getShelfLife());
                inputsInfoDOs.add(inputsInfoDO);

                relationalDO.setInputsId(inputsInfoDO.getInputsId());
            } else {
                // 绑定旧的投入品
                InputsInfoDO inputsInfoDO = inputsInfoMapper.selectOne(new LambdaQueryWrapperX<InputsInfoDO>().eq(InputsInfoDO::getInputsId, item.getInputsId()));
                if (inputsInfoDO == null) {
                    throw exception(ErrorCodeConstants.INPUTS_INFO_NOT_EXISTS);
                }

                relationalDO.setInputsId(inputsInfoDO.getInputsId());
            }
            goodsInputRelationalDOs.add(relationalDO);
        }

        // 批量插入新增的投入品
        inputsInfoMapper.insertBatch(inputsInfoDOs);

        goodsInputRelationalMapper.insertBatch(goodsInputRelationalDOs);
    }
}
