package cn.iocoder.yudao.module.basic.service.inputsinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.*;
import cn.iocoder.yudao.module.basic.dal.dataobject.inputsinfo.InputsInfoDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 投入品 Service 接口
 *
 * @author 芋道源码
 */
public interface InputsInfoService {

    /**
     * 创建投入品
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInputsInfo(@Valid InputsInfoCreateReqVO createReqVO);

    /**
     * 更新投入品
     *
     * @param updateReqVO 更新信息
     */
    void updateInputsInfo(@Valid InputsInfoUpdateReqVO updateReqVO);

    /**
     * 删除投入品
     *
     * @param id 编号
     */
    void deleteInputsInfo(Long id);

    /**
     * 获得投入品
     *
     * @param id 编号
     * @return 投入品
     */
    InputsInfoDO getInputsInfo(Long id);

    /**
     * 获得投入品分页
     *
     * @param pageReqVO 分页查询
     * @return 投入品分页
     */
    PageResult<InputsInfoDO> getInputsInfoPage(InputsInfoPageReqVO pageReqVO);

    /**
     * 获得投入品列表
     *
     * @param exportReqVO 查询条件
     * @return 投入品列表
     */
    List<InputsInfoDO> getInputsInfoList(InputsInfoExportReqVO exportReqVO);

    /**
     * 获取所有投入品
     * @return
     */
    List<InputsInfoDO> getInputsInfoInputsInfoList();

    /**
     * 根据指定的商户id 获取所有名下多个投入品
     * @param belongBusinessId
     * @return
     */
    List<InputsInfoDO> getInputsInfoListByBelongBusinessId(List<Long> belongBusinessId);

    /**
     * 根据商户ID 获取所属投入品
     * @param businessInfoId 商户ID
     * @return 投入品
     */
    List<InputsInfoRespVO> getInputsInfoByBusinessInfoId(Long businessInfoId);

    /**
     * 根据商户ID获得投入品列表
     * @param businessInfoId 商户ID
     * @return 投入品
     */
    List<InputsInfoForTransferRespVO> queryListOfMiniResult(Long businessInfoId);
}
