package cn.iocoder.yudao.module.basic.service.inputsinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.InputsInfoCreateReqVO;
import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.InputsInfoExportReqVO;
import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.InputsInfoPageReqVO;
import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.InputsInfoUpdateReqVO;
import cn.iocoder.yudao.module.basic.dal.dataobject.inputsinfo.InputsInfoDO;
import cn.iocoder.yudao.module.basic.dal.mysql.inputsinfo.InputsInfoMapper;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.cloneIgnoreId;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.INPUTS_INFO_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.*;

/**
* {@link InputsInfoServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(InputsInfoServiceImpl.class)
public class InputsInfoServiceImplTest extends BaseDbUnitTest {

    @Resource
    private InputsInfoServiceImpl inputsInfoService;

    @Resource
    private InputsInfoMapper inputsInfoMapper;

    @Test
    public void testCreateInputsInfo_success() {
        // 准备参数
        InputsInfoCreateReqVO reqVO = randomPojo(InputsInfoCreateReqVO.class);

        // 调用
        Long inputsInfoId = inputsInfoService.createInputsInfo(reqVO);
        // 断言
        assertNotNull(inputsInfoId);
        // 校验记录的属性是否正确
        InputsInfoDO inputsInfo = inputsInfoMapper.selectById(inputsInfoId);
        assertPojoEquals(reqVO, inputsInfo);
    }

    @Test
    public void testUpdateInputsInfo_success() {
        // mock 数据
        InputsInfoDO dbInputsInfo = randomPojo(InputsInfoDO.class);
        inputsInfoMapper.insert(dbInputsInfo);// @Sql: 先插入出一条存在的数据
        // 准备参数
        InputsInfoUpdateReqVO reqVO = randomPojo(InputsInfoUpdateReqVO.class, o -> {
            o.setId(dbInputsInfo.getId()); // 设置更新的 ID
        });

        // 调用
        inputsInfoService.updateInputsInfo(reqVO);
        // 校验是否更新正确
        InputsInfoDO inputsInfo = inputsInfoMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, inputsInfo);
    }

    @Test
    public void testUpdateInputsInfo_notExists() {
        // 准备参数
        InputsInfoUpdateReqVO reqVO = randomPojo(InputsInfoUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> inputsInfoService.updateInputsInfo(reqVO), INPUTS_INFO_NOT_EXISTS);
    }

    @Test
    public void testDeleteInputsInfo_success() {
        // mock 数据
        InputsInfoDO dbInputsInfo = randomPojo(InputsInfoDO.class);
        inputsInfoMapper.insert(dbInputsInfo);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbInputsInfo.getId();

        // 调用
        inputsInfoService.deleteInputsInfo(id);
       // 校验数据不存在了
       assertNull(inputsInfoMapper.selectById(id));
    }

    @Test
    public void testDeleteInputsInfo_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> inputsInfoService.deleteInputsInfo(id), INPUTS_INFO_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetInputsInfoPage() {
       // mock 数据
       InputsInfoDO dbInputsInfo = randomPojo(InputsInfoDO.class, o -> { // 等会查询到
           o.setInputsName(null);
           o.setInputsId(null);
           o.setInputsUnitId(null);
           o.setConversionValue(null);
           o.setIsBaseUnit(null);
           o.setInputsSpecs(null);
           o.setInputsRemarks(null);
           o.setInputsPhotoUrl(null);
           o.setShelfLife(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       inputsInfoMapper.insert(dbInputsInfo);
       // 测试 inputsName 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsName(null)));
       // 测试 inputsId 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsId(null)));
       // 测试 inputsUnit 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsUnitId(null)));
        // 测试 inputsUnit 不匹配
        inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setConversionValue(null)));
        // 测试 inputsUnit 不匹配
        inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setIsBaseUnit(null)));
       // 测试 inputsSpecs 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsSpecs(null)));
       // 测试 inputsRemarks 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsRemarks(null)));
       // 测试 inputsPhotoUrl 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsPhotoUrl(null)));
       // 测试 shelfLife 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setShelfLife(null)));
       // 测试 belongBusinessId 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setCreateTime(null)));
       // 准备参数
       InputsInfoPageReqVO reqVO = new InputsInfoPageReqVO();
       reqVO.setInputsName(null);
       reqVO.setInputsId(null);
       reqVO.setInputsUnitId(null);
       reqVO.setConversionValue(null);
       reqVO.setIsBaseUnit(null);
       reqVO.setInputsSpecs(null);
       reqVO.setInputsRemarks(null);
       reqVO.setInputsPhotoUrl(null);
       reqVO.setShelfLife(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<InputsInfoDO> pageResult = inputsInfoService.getInputsInfoPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbInputsInfo, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetInputsInfoList() {
       // mock 数据
       InputsInfoDO dbInputsInfo = randomPojo(InputsInfoDO.class, o -> { // 等会查询到
           o.setInputsName(null);
           o.setInputsId(null);
           o.setInputsUnitId(null);
           o.setConversionValue(null);
           o.setIsBaseUnit(null);
           o.setInputsSpecs(null);
           o.setInputsRemarks(null);
           o.setInputsPhotoUrl(null);
           o.setShelfLife(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       inputsInfoMapper.insert(dbInputsInfo);
       // 测试 inputsName 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsName(null)));
       // 测试 inputsId 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsId(null)));
        // 测试 inputsUnit 不匹配
        inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsUnitId(null)));
        // 测试 inputsUnit 不匹配
        inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setConversionValue(null)));
        // 测试 inputsUnit 不匹配
        inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setIsBaseUnit(null)));
       // 测试 inputsSpecs 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsSpecs(null)));
       // 测试 inputsRemarks 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsRemarks(null)));
       // 测试 inputsPhotoUrl 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setInputsPhotoUrl(null)));
       // 测试 shelfLife 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setShelfLife(null)));
       // 测试 belongBusinessId 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       inputsInfoMapper.insert(cloneIgnoreId(dbInputsInfo, o -> o.setCreateTime(null)));
       // 准备参数
       InputsInfoExportReqVO reqVO = new InputsInfoExportReqVO();
       reqVO.setInputsName(null);
       reqVO.setInputsId(null);
        reqVO.setInputsUnitId(null);
        reqVO.setConversionValue(null);
        reqVO.setIsBaseUnit(null);
       reqVO.setInputsSpecs(null);
       reqVO.setInputsRemarks(null);
       reqVO.setInputsPhotoUrl(null);
       reqVO.setShelfLife(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       List<InputsInfoDO> list = inputsInfoService.getInputsInfoList(reqVO);
       // 断言
       assertEquals(1, list.size());
       assertPojoEquals(dbInputsInfo, list.get(0));
    }

}
