package cn.iocoder.yudao.module.configure.controller.admin.tracecert;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.configure.controller.admin.tracecert.vo.*;
import cn.iocoder.yudao.module.configure.dal.dataobject.tracecert.TraceCertDO;
import cn.iocoder.yudao.module.configure.convert.tracecert.TraceCertConvert;
import cn.iocoder.yudao.module.configure.service.tracecert.TraceCertService;

@Api(tags = "管理后台 - 溯源链证书配置")
@RestController
@RequestMapping("/configure/trace-cert")
@Validated
public class TraceCertController {

    @Resource
    private TraceCertService traceCertService;

    @PostMapping("/create")
    @ApiOperation("创建溯源链证书配置")
    @PreAuthorize("@ss.hasPermission('configure:trace-cert:create')")
    public CommonResult<Long> createTraceCert(@Valid @RequestBody TraceCertCreateReqVO createReqVO) {
        return success(traceCertService.createTraceCert(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新溯源链证书配置")
    @PreAuthorize("@ss.hasPermission('configure:trace-cert:update')")
    public CommonResult<Boolean> updateTraceCert(@Valid @RequestBody TraceCertUpdateReqVO updateReqVO) {
        traceCertService.updateTraceCert(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除溯源链证书配置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('configure:trace-cert:delete')")
    public CommonResult<Boolean> deleteTraceCert(@RequestParam("id") Long id) {
        traceCertService.deleteTraceCert(id);
        return success(true);
    }

    @GetMapping("/getByBelongBusinessId")
    @ApiOperation("根据所属商户id,获得溯源链证书配置信息")
    @ApiImplicitParam(name = "belongBusinessId", value = "所属商户id", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('configure:trace-cert:query')")
    public CommonResult<TraceCertRespVO> getByBelongBusinessId(@RequestParam("belongBusinessId") Long belongBusinessId) {
        TraceCertDO traceCert = traceCertService.getTraceCertByBusinessId(belongBusinessId);
        return success(TraceCertConvert.INSTANCE.convert(traceCert));
    }

}
