package cn.iocoder.yudao.module.configure.service.tracecert;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.module.configure.controller.admin.tracecert.vo.*;
import cn.iocoder.yudao.module.configure.dal.dataobject.tracecert.TraceCertDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.configure.convert.tracecert.TraceCertConvert;
import cn.iocoder.yudao.module.configure.dal.mysql.tracecert.TraceCertMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.configure.enums.ErrorCodeConstants.*;

/**
 * 溯源链证书配置 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class TraceCertServiceImpl implements TraceCertService {

    @Resource
    private TraceCertMapper traceCertMapper;

    @Override
    public Long createTraceCert(TraceCertCreateReqVO createReqVO) {
        // 校验存在 所属商户对应的溯源链证书配置信息是否存在 如果存在则不能再进行创建
        validateTraceCertExists((long)0,createReqVO.getBelongBusinessId());
        // 插入
        TraceCertDO traceCert = TraceCertConvert.INSTANCE.convert(createReqVO);
        traceCertMapper.insert(traceCert);
        // 返回
        return traceCert.getId();
    }

    @Override
    public void updateTraceCert(TraceCertUpdateReqVO updateReqVO) {
        // 校验存在
        validateTraceCertExists(updateReqVO.getId(),(long)0);
        // 更新
        TraceCertDO updateObj = TraceCertConvert.INSTANCE.convert(updateReqVO);
        traceCertMapper.updateById(updateObj);
    }

    @Override
    public void deleteTraceCert(Long id) {
        // 校验存在
        validateTraceCertExists(id,(long)0);
        // 删除
        traceCertMapper.deleteById(id);
    }

    /**
     * 获得溯源链证书配置
     *
     * @param belongBusinessId 所属商户id
     * @return 溯源链证书配置
     */
    @Override
    public TraceCertDO getTraceCertByBusinessId(Long belongBusinessId) {
        return traceCertMapper.selectOne(TraceCertDO::getBelongBusinessId,belongBusinessId);
    }

    private void validateTraceCertExists(Long id,Long belongBusinessId) {
        if(id != 0){
            if (traceCertMapper.selectById(id) == null) {
                throw exception(TRACE_CERT_NOT_EXISTS);
            }
        }
        if(belongBusinessId != 0){
            if (traceCertMapper.selectOne(TraceCertDO::getBelongBusinessId,belongBusinessId) != null) {
                throw exception(TRACE_CERT_IS_EXISTS_BY_BELONG_BUSINESS);
            }
        }
    }

}
