package cn.iocoder.yudao.module.identity.service.businessrule;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.identity.controller.admin.businessrule.vo.*;
import cn.iocoder.yudao.module.identity.dal.dataobject.businessrule.BusinessRuleDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 码规则 Service 接口
 *
 * @author 芋道源码
 */
public interface BusinessRuleService {

    /**
     * 创建码规则
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBusinessRule(@Valid BusinessRuleCreateReqVO createReqVO);

    /**
     * 更新码规则
     *
     * @param updateReqVO 更新信息
     */
    void updateBusinessRule(@Valid BusinessRuleUpdateReqVO updateReqVO);

    /**
     * 删除码规则
     *
     * @param id 编号
     */
    void deleteBusinessRule(Long id);

    /**
     * 获得码规则
     *
     * @param id 编号
     * @return 码规则
     */
    BusinessRuleDO getBusinessRule(Long id);

    /**
     * 获得码规则列表
     *
     * @param ids 编号
     * @return 码规则列表
     */
    List<BusinessRuleDO> getBusinessRuleList(Collection<Long> ids);

    /**
     * 获得码规则列表
     *
     * @param businessId 商户编编码
     * @return 码规则列表
     */
    List<BusinessRuleDO> getBusinessRuleListByBusinessId(String businessId);

    /**
     * 获得码规则分页
     *
     * @param pageReqVO 分页查询
     * @return 码规则分页
     */
    PageResult<BusinessRuleDO> getBusinessRulePage(BusinessRulePageReqVO pageReqVO);

    /**
     * 根据商户昵称获取马规则信息
     * @param businessName
     * @return
     */
    BusinessRuleDO getBusinessRuleByBusinessName(String businessName);

}
