package cn.iocoder.yudao.module.product.controller.admin.inputsbatch;

import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;

import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbatch.InputsBatchDO;
import cn.iocoder.yudao.module.product.convert.inputsbatch.InputsBatchConvert;
import cn.iocoder.yudao.module.product.service.inputsbatch.InputsBatchService;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 投入品批次")
@RestController
@RequestMapping("/product/inputs-batch")
@Validated
public class InputsBatchController {

    @Resource
    private InputsBatchService inputsBatchService;

    @PostMapping("/create")
    @ApiOperation("创建投入品批次")
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:create')")
    public CommonResult<Long> createInputsBatch(@Valid @RequestBody InputsBatchCreateReqVO createReqVO) {
        return CommonResult.success(inputsBatchService.createInputsBatch(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新投入品批次")
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:update')")
    public CommonResult<Boolean> updateInputsBatch(@Valid @RequestBody InputsBatchSaveReqVO updateReqVO) {
        inputsBatchService.updateInputsBatch(updateReqVO);
        return CommonResult.success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除投入品批次")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:delete')")
    public CommonResult<Boolean> deleteInputsBatch(@RequestParam("id") Long id) {
        inputsBatchService.deleteInputsBatch(id);
        return CommonResult.success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得投入品批次")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:query')")
    public CommonResult<InputsBatchRespVO> getInputsBatch(@RequestParam("id") Long id) {
        InputsBatchDO inputsBatch = inputsBatchService.getInputsBatch(id);
        return CommonResult.success(InputsBatchConvert.INSTANCE.convert(inputsBatch));
    }

    @GetMapping("/list")
    @ApiOperation("获得投入品批次列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:query')")
    public CommonResult<List<InputsBatchRespVO>> getInputsBatchList(@RequestParam("ids") Collection<Long> ids) {
        List<InputsBatchDO> list = inputsBatchService.getInputsBatchList(ids);
        return CommonResult.success(InputsBatchConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得投入品批次分页")
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:query')")
    public CommonResult<PageResult<InputsBatchRespVO>> getInputsBatchPage(@Valid InputsBatchPageReqVO pageVO) {
        PageResult<InputsBatchDO> pageResult = inputsBatchService.getInputsBatchPage(pageVO);
        return CommonResult.success(InputsBatchConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出投入品批次 Excel")
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:export')")
    @OperateLog(type = OperateTypeEnum.EXPORT)
    public void exportInputsBatchExcel(@Valid InputsBatchExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<InputsBatchDO> list = inputsBatchService.getInputsBatchList(exportReqVO);
        // 导出 Excel
        List<InputsBatchExcelVO> datas = InputsBatchConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "投入品批次.xls", "数据", InputsBatchExcelVO.class, datas);
    }

    @GetMapping("/getInputsBatchNumberList")
    @ApiOperation("获得投入品批次编号列表")
    @ApiImplicitParam(name = "count", value = "数量", required = true, example = "1", dataTypeClass = Integer.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:query')")
    public CommonResult<List<String>> getInputsBatchNumberList(@RequestParam("count") @NotNull Integer count) {
        return CommonResult.success(inputsBatchService.getInputsBatchNumberList(count));
    }

    @DeleteMapping("/deleteBatch")
    @ApiOperation("批量删除投入品批次")
    @ApiImplicitParam(name = "ids", value = "编号集合", required = true, dataTypeClass = Arrays.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:delete')")
    public CommonResult<Boolean> deleteBatch(@RequestParam("ids") @NotEmpty List<Long> ids) {
        inputsBatchService.deleteBatch(ids);
        return CommonResult.success(true);
    }

    @GetMapping("/getInputsBatchInputsBatchIdList")
    @ApiOperation("根据投入品id获取所有投入品批次")
    @ApiImplicitParam(name = "inputsId", value = "投入品id", required = true, example = "1", dataTypeClass = Integer.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:query')")
    public CommonResult<List<String>> getInputsBatchInputsBatchIdList(@RequestParam("inputsId") String inputsId) {
        List<String> inputsBatchInputsBatchIdList = inputsBatchService.getInputsBatchInputsBatchIdList(inputsId);
        return CommonResult.success(inputsBatchInputsBatchIdList);
    }

    @PutMapping("/freeze")
    @ApiOperation("冻结")
    @ApiImplicitParam(name = "id", value = "ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:freeze')")
    public CommonResult<Boolean> freeze(@RequestParam("id") Long id) {
        inputsBatchService.updateChainStatus(id, BlockStateEnum.FREEZE.getBlockState());
        return success(Boolean.TRUE);
    }

    @PutMapping("/active")
    @ApiOperation("激活")
    @ApiImplicitParam(name = "id", value = "ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-batch:active')")
    public CommonResult<Boolean> active(@RequestParam("id") Long id) {
        inputsBatchService.updateChainStatus(id, BlockStateEnum.ACTIVATION.getBlockState());
        return success(Boolean.TRUE);
    }

}
