package cn.iocoder.yudao.module.product.controller.admin.inputsbuy;

import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyPageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyRespVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.inputsbuy.InputsBuyConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbuy.InputsBuyDO;
import cn.iocoder.yudao.module.product.service.inputsbuy.InputsBuyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Api(tags = "管理后台 - 投入品采购")
@RestController
@RequestMapping("/product/inputs-buy")
@Validated
public class InputsBuyController {

    @Resource
    private InputsBuyService inputsBuyService;

    @PostMapping("/create")
    @ApiOperation("创建投入品采购")
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:create')")
    public CommonResult<Long> createInputsBuy(@Valid @RequestBody InputsBuyCreateReqVO createReqVO) {
        return CommonResult.success(inputsBuyService.createInputsBuy(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新投入品采购")
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:update')")
    public CommonResult<Boolean> updateInputsBuy(@Valid @RequestBody InputsBuyUpdateReqVO updateReqVO) {
        inputsBuyService.updateInputsBuy(updateReqVO);
        return CommonResult.success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除投入品采购")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:delete')")
    public CommonResult<Boolean> deleteInputsBuy(@RequestParam("id") Long id) {
        inputsBuyService.deleteInputsBuy(id);
        return CommonResult.success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得投入品采购")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:query')")
    public CommonResult<InputsBuyRespVO> getInputsBuy(@RequestParam("id") Long id) {
        return CommonResult.success(inputsBuyService.getInputsBuy(id));
    }

    @GetMapping("/list")
    @ApiOperation("获得投入品采购列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:query')")
    public CommonResult<List<InputsBuyRespVO>> getInputsBuyList(@RequestParam("ids") Collection<Long> ids) {
        List<InputsBuyDO> list = inputsBuyService.getInputsBuyList(ids);
        return CommonResult.success(InputsBuyConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得投入品采购分页")
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:query')")
    public CommonResult<PageResult<InputsBuyRespVO>> getInputsBuyPage(@Valid InputsBuyPageReqVO pageVO) {
        return CommonResult.success(inputsBuyService.getInputsBuyPage(pageVO));
    }

    @GetMapping("/getInputsBuyNumberList")
    @ApiOperation("获得投入品批次编号列表")
    @ApiImplicitParam(name = "count", value = "数量", required = true, example = "1", dataTypeClass = Integer.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:query')")
    public CommonResult<List<String>> getInputsBuyNumberList(@RequestParam("count") @NotNull Integer count) {
        return CommonResult.success(inputsBuyService.getInputsBuyNumberList(count));
    }

    @GetMapping("/getInputsBuyByOutbound")
    @ApiOperation("获得投入品采购（通过出库单号，用于新增）")
    @ApiImplicitParam(name = "outBoundId", value = "出库单号", required = true, example = "OUPJOE20230420153938958B81W0", dataTypeClass = String.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:query')")
    public CommonResult<Object> getInputsBuyByOutbound(@RequestParam("outBoundId") String outBoundId) {
        return CommonResult.success(inputsBuyService.getInputsBuyByOutbound(outBoundId));
    }

    @PutMapping("/upChain")
    @ApiOperation("上链")
    @ApiImplicitParam(name = "id", value = "id", required = true, example = "1", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:upChain')")
    public CommonResult<Boolean> upChain(@RequestParam("id") Long id) {
        inputsBuyService.upChain(id);
        return CommonResult.success(true);
    }

    @PutMapping("/freeze")
    @ApiOperation("冻结")
    @ApiImplicitParam(name = "id", value = "ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:freeze')")
    public CommonResult<Boolean> freeze(@RequestParam("id") Long id) {
        inputsBuyService.updateChainStatus(id, BlockStateEnum.FREEZE.getBlockState());
        return success(Boolean.TRUE);
    }

    @PutMapping("/active")
    @ApiOperation("激活")
    @ApiImplicitParam(name = "id", value = "ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inputs-buy:active')")
    public CommonResult<Boolean> active(@RequestParam("id") Long id) {
        inputsBuyService.updateChainStatus(id, BlockStateEnum.ACTIVATION.getBlockState());
        return success(Boolean.TRUE);
    }

}
