package cn.iocoder.yudao.module.product.controller.admin.landblockstate;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStatePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landblockstate.LandBlockStateConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landblockstate.LandBlockStateDO;
import cn.iocoder.yudao.module.product.service.landblockstate.LandBlockStateService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 地块状态")
@RestController
@RequestMapping("/product/land-block-state")
@Validated
public class LandBlockStateController {

    @Resource
    private LandBlockStateService landBlockStateService;

    @PostMapping("/create")
    @ApiOperation("创建地块状态")
    @PreAuthorize("@ss.hasPermission('product:land-block-state:create')")
    public CommonResult<Long> createLandBlockState(@Valid @RequestBody LandBlockStateCreateReqVO createReqVO) {
        return success(landBlockStateService.createLandBlockState(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新地块状态")
    @PreAuthorize("@ss.hasPermission('product:land-block-state:update')")
    public CommonResult<Boolean> updateLandBlockState(@Valid @RequestBody LandBlockStateUpdateReqVO updateReqVO) {
        landBlockStateService.updateLandBlockState(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除地块状态")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-block-state:delete')")
    public CommonResult<Boolean> deleteLandBlockState(@RequestParam("id") Long id) {
        landBlockStateService.deleteLandBlockState(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得地块状态")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-block-state:query')")
    public CommonResult<LandBlockStateRespVO> getLandBlockState(@RequestParam("id") Long id) {
        LandBlockStateDO landBlockState = landBlockStateService.getLandBlockState(id);
        return success(LandBlockStateConvert.INSTANCE.convert(landBlockState));
    }

    @GetMapping("/list")
    @ApiOperation("获得地块状态列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:land-block-state:query')")
    public CommonResult<List<LandBlockStateRespVO>> getLandBlockStateList(@RequestParam("ids") Collection<Long> ids) {
        List<LandBlockStateDO> list = landBlockStateService.getLandBlockStateList(ids);
        return success(LandBlockStateConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得地块状态分页")
    @PreAuthorize("@ss.hasPermission('product:land-block-state:query')")
    public CommonResult<PageResult<LandBlockStateRespVO>> getLandBlockStatePage(@Valid LandBlockStatePageReqVO pageVO) {
        PageResult<LandBlockStateDO> pageResult = landBlockStateService.getLandBlockStatePage(pageVO);
        return success(LandBlockStateConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getBusinessInfoByListLandBlockState")
    @ApiOperation("根据商户ID信息获取所有地块状态")
    @ApiImplicitParam(name = "id", value = "商户信息ID", required = true, example = "1", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-info-type:query')")
    public CommonResult<List<LandBlockStateRespVO>> getBusinessInfoByListLandBlockState(@RequestParam("id") Long id) {
        List<LandBlockStateDO> byBusinessInfoId = landBlockStateService.getByBusinessInfoId(id);
        return success(LandBlockStateConvert.INSTANCE.convertList(byBusinessInfoId));
    }

}
