package cn.iocoder.yudao.module.product.controller.admin.landinfotype;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landinfotype.LandInfoTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfotype.LandInfoTypeDO;
import cn.iocoder.yudao.module.product.service.landinfotype.LandInfoTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 地块类型")
@RestController
@RequestMapping("/product/land-info-type")
@Validated
public class LandInfoTypeController {

    @Resource
    private LandInfoTypeService landInfoTypeService;

    @PostMapping("/create")
    @ApiOperation("创建地块类型")
    @PreAuthorize("@ss.hasPermission('product:land-info-type:create')")
    public CommonResult<Long> createLandInfoType(@Valid @RequestBody LandInfoTypeCreateReqVO createReqVO) {
        return success(landInfoTypeService.createLandInfoType(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新地块类型")
    @PreAuthorize("@ss.hasPermission('product:land-info-type:update')")
    public CommonResult<Boolean> updateLandInfoType(@Valid @RequestBody LandInfoTypeUpdateReqVO updateReqVO) {
        landInfoTypeService.updateLandInfoType(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除地块类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-info-type:delete')")
    public CommonResult<Boolean> deleteLandInfoType(@RequestParam("id") Long id) {
        landInfoTypeService.deleteLandInfoType(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得地块类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-info-type:query')")
    public CommonResult<LandInfoTypeRespVO> getLandInfoType(@RequestParam("id") Long id) {
        LandInfoTypeDO landInfoType = landInfoTypeService.getLandInfoType(id);
        return success(LandInfoTypeConvert.INSTANCE.convert(landInfoType));
    }

    @GetMapping("/list")
    @ApiOperation("获得地块类型列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:land-info-type:query')")
    public CommonResult<List<LandInfoTypeRespVO>> getLandInfoTypeList(@RequestParam("ids") Collection<Long> ids) {
        List<LandInfoTypeDO> list = landInfoTypeService.getLandInfoTypeList(ids);
        return success(LandInfoTypeConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得地块类型分页")
    @PreAuthorize("@ss.hasPermission('product:land-info-type:query')")
    public CommonResult<PageResult<LandInfoTypeRespVO>> getLandInfoTypePage(@Valid LandInfoTypePageReqVO pageVO) {
        PageResult<LandInfoTypeDO> pageResult = landInfoTypeService.getLandInfoTypePage(pageVO);
        return success(LandInfoTypeConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getByBusinessInfoId")
    @ApiOperation("根据商户ID信息获取所有地块类型")
    @ApiImplicitParam(name = "id", value = "商户信息ID", required = true, example = "1", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-info-type:query')")
    public CommonResult<List<LandInfoTypeRespVO>> getByBusinessInfoId(@RequestParam("id") Long id) {
        List<LandInfoTypeDO> byBusinessInfoId = landInfoTypeService.getByBusinessInfoId(id);
        return success(LandInfoTypeConvert.INSTANCE.convertList(byBusinessInfoId));
    }

}
