package cn.iocoder.yudao.module.product.controller.admin.landjobinputs;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsPageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landjobinputs.LandJobInputsConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobinputs.LandJobInputsDO;
import cn.iocoder.yudao.module.product.service.landjobinputs.LandJobInputsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 地块作业投入品")
@RestController
@RequestMapping("/product/land-job-inputs")
@Validated
public class LandJobInputsController {

    @Resource
    private LandJobInputsService landJobInputsService;

    @PostMapping("/create")
    @ApiOperation("创建地块作业投入品")
    @PreAuthorize("@ss.hasPermission('product:land-job-inputs:create')")
    public CommonResult<Long> createLandJobInputs(@Valid @RequestBody LandJobInputsCreateReqVO createReqVO) {
        return success(landJobInputsService.createLandJobInputs(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新地块作业投入品")
    @PreAuthorize("@ss.hasPermission('product:land-job-inputs:update')")
    public CommonResult<Boolean> updateLandJobInputs(@Valid @RequestBody LandJobInputsUpdateReqVO updateReqVO) {
        landJobInputsService.updateLandJobInputs(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除地块作业投入品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job-inputs:delete')")
    public CommonResult<Boolean> deleteLandJobInputs(@RequestParam("id") Long id) {
        landJobInputsService.deleteLandJobInputs(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得地块作业投入品")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job-inputs:query')")
    public CommonResult<LandJobInputsRespVO> getLandJobInputs(@RequestParam("id") Long id) {
        LandJobInputsDO landJobInputs = landJobInputsService.getLandJobInputs(id);
        return success(LandJobInputsConvert.INSTANCE.convert(landJobInputs));
    }

    @GetMapping("/list")
    @ApiOperation("获得地块作业投入品列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:land-job-inputs:query')")
    public CommonResult<List<LandJobInputsRespVO>> getLandJobInputsList(@RequestParam("ids") Collection<Long> ids) {
        List<LandJobInputsDO> list = landJobInputsService.getLandJobInputsList(ids);
        return success(LandJobInputsConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得地块作业投入品分页")
    @PreAuthorize("@ss.hasPermission('product:land-job-inputs:query')")
    public CommonResult<PageResult<LandJobInputsRespVO>> getLandJobInputsPage(@Valid LandJobInputsPageReqVO pageVO) {
        PageResult<LandJobInputsDO> pageResult = landJobInputsService.getLandJobInputsPage(pageVO);
        return success(LandJobInputsConvert.INSTANCE.convertPage(pageResult));
    }

}
