package cn.iocoder.yudao.module.product.controller.admin.landjobtype;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landjobtype.LandJobTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobtype.LandJobTypeDO;
import cn.iocoder.yudao.module.product.service.landjobtype.LandJobTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 作业类型")
@RestController
@RequestMapping("/product/land-job-type")
@Validated
public class LandJobTypeController {

    @Resource
    private LandJobTypeService landJobTypeService;

    @PostMapping("/create")
    @ApiOperation("创建作业类型")
    @PreAuthorize("@ss.hasPermission('product:land-job-type:create')")
    public CommonResult<Long> createLandJobType(@Valid @RequestBody LandJobTypeCreateReqVO createReqVO) {
        return success(landJobTypeService.createLandJobType(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新作业类型")
    @PreAuthorize("@ss.hasPermission('product:land-job-type:update')")
    public CommonResult<Boolean> updateLandJobType(@Valid @RequestBody LandJobTypeUpdateReqVO updateReqVO) {
        landJobTypeService.updateLandJobType(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除作业类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job-type:delete')")
    public CommonResult<Boolean> deleteLandJobType(@RequestParam("id") Long id) {
        landJobTypeService.deleteLandJobType(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得作业类型")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job-type:query')")
    public CommonResult<LandJobTypeRespVO> getLandJobType(@RequestParam("id") Long id) {
        LandJobTypeDO landJobType = landJobTypeService.getLandJobType(id);
        return success(LandJobTypeConvert.INSTANCE.convert(landJobType));
    }

    @GetMapping("/page")
    @ApiOperation("获得作业类型分页")
    @PreAuthorize("@ss.hasPermission('product:land-job-type:query')")
    public CommonResult<PageResult<LandJobTypeRespVO>> getLandJobTypePage(@Valid LandJobTypePageReqVO pageVO) {
        PageResult<LandJobTypeDO> pageResult = landJobTypeService.getLandJobTypePage(pageVO);
        return success(LandJobTypeConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getByBusinessInfoId")
    @ApiOperation("根据商户ID获得作业类型")
    @ApiImplicitParam(name = "id", value = "商户信息ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job-type:query')")
    public CommonResult<List<LandJobTypeRespVO>> getByBusinessInfoId(@RequestParam("id") Long id) {
        List<LandJobTypeDO> landJobTypes = landJobTypeService.getByBusinessInfoId(id);
        return success(LandJobTypeConvert.INSTANCE.convertList(landJobTypes));
    }


}
