package cn.iocoder.yudao.module.product.controller.admin.landplan;

import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.FarmingPlantReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landplan.vo.*;
import cn.iocoder.yudao.module.product.convert.landplan.LandPlanConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landplan.LandPlanDO;
import cn.iocoder.yudao.module.product.service.landplan.LandPlanService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 种植计划")
@RestController
@RequestMapping("/product/land-plan")
@Validated
public class LandPlanController {

    @Resource
    private LandPlanService landPlanService;

    @PostMapping("/create")
    @ApiOperation("创建地块作业计划")
//    @PreAuthorize("@ss.hasPermission('product:land-plan:create')")
    public CommonResult<Long> createLandPlan(@Valid @RequestBody LandPlanCreateReqVO createReqVO) {
        Long landPlan = landPlanService.createLandPlan(createReqVO);
        return success(landPlan);
    }

    @PostMapping("/createList")
    @ApiOperation("批量创建地块作业计划")
    @PreAuthorize("@ss.hasPermission('product:land-plan:create')")
    public CommonResult<List<Long>> createLandPlanList(@Valid @RequestBody List<LandPlanCreateReqVO> createReqVOList) {
        List<Long> landPlanList = landPlanService.createLandPlanList(createReqVOList);
        return success(landPlanList);
    }

    @PutMapping("/update")
    @ApiOperation("更新地块作业计划")
    @PreAuthorize("@ss.hasPermission('product:land-plan:update')")
    public CommonResult<Boolean> updateLandPlan(@Valid @RequestBody LandPlanUpdateReqVO updateReqVO) {
        landPlanService.updateLandPlan(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除地块作业计划")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-plan:delete')")
    public CommonResult<Boolean> deleteLandPlan(@RequestParam("id") Long id) {
        landPlanService.deleteLandPlan(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得地块作业计划")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-plan:query')")
    public CommonResult<LandPlanRespVO> getLandPlan(@RequestParam("id") Long id) {
        LandPlanDO landPlan = landPlanService.getLandPlan(id);
        return success(LandPlanConvert.INSTANCE.convert(landPlan));
    }

    @GetMapping("/getLandPlanByGoodsId")
    @ApiOperation("获得地块作业计划列表")
    @ApiImplicitParam(name = "goodsId", value = "商品编码", required = true, example = "GOCD202304131028TSOC", dataTypeClass = String.class)
    @PreAuthorize("@ss.hasPermission('product:land-plan:query')")
    public CommonResult<List<LandPlanRespVO>> getLandPlanByGoodsId(@RequestParam("goodsId") String goodsId) {
        List<LandPlanDO> list = landPlanService.getLandPlanByGoodsId(goodsId);
        return success(LandPlanConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得地块作业计划分页")
    @PreAuthorize("@ss.hasPermission('product:land-plan:query')")
    public CommonResult<PageResult<LandPlanRespVO>> getLandPlanPage(@Valid LandPlanPageReqVO pageVO) {
        PageResult<LandPlanDO> pageResult = landPlanService.getLandPlanPage(pageVO);
        return success(LandPlanConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getLandPlanForFarmingConfig")
    @ApiOperation("获得地块作业计划列表，用于农事活动配置")
    @PreAuthorize("@ss.hasPermission('product:land-plan:query')")
    public CommonResult<List<FarmingPlantReqVO>> getLandPlanForFarmingConfig(LandPlanReqVO params) {
        return success(landPlanService.getLandPlanForFarmingConfig(params.getGoodsId(), params.getIds(), params.getPlanIds(), BlockStateEnum.ACTIVATION.getBlockState()));
    }

}
