package cn.iocoder.yudao.module.product.controller.app.landjob;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.LandJobCreateOfAppReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.LandJobRespVO;
import cn.iocoder.yudao.module.product.controller.app.landjob.vo.FileUploadReqVO;
import cn.iocoder.yudao.module.product.controller.app.landjob.vo.LandJobReqVO;
import cn.iocoder.yudao.module.product.service.landjob.LandJobService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "APP移动端 - 种植作业")
@RestController
@RequestMapping("/product/land-job")
@Validated
public class LandJobAppController {

    @Resource
    private LandJobService landJobService;

    @GetMapping("/getSameTypeJobById")
    @PermitAll // 无需登陆
    @ApiOperation("获得同类型的农事作业")
    public CommonResult<List<LandJobRespVO>> getSameTypeJobById(@Valid LandJobReqVO landJobReqVO) {
        return CommonResult.success(landJobService.getSameTypeJobById(landJobReqVO));
    }

    @PostMapping("/upload")
    @PermitAll
    @ApiOperation("上传文件")
    @OperateLog(logArgs = false) // 上传文件，没有记录操作日志的必要
    public CommonResult<String> uploadFile(@Valid FileUploadReqVO uploadReqVO) throws Exception {
        return success(landJobService.upload(uploadReqVO));
    }

    @PostMapping("/create")
    @PermitAll // 无需登陆
    @ApiOperation("创建种植/养殖/畜牧作业")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<Long> create(@Valid @RequestBody LandJobCreateOfAppReqVO createReqVO) {
        return CommonResult.success(landJobService.createLandJobByApp(createReqVO));
    }

}
