package cn.iocoder.yudao.module.product.service.inputsbatch;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbatch.InputsBatchDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 投入品批次 Service 接口
 *
 * @author 芋道源码
 */
public interface InputsBatchService {

    /**
     * 创建投入品批次
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInputsBatch(@Valid InputsBatchCreateReqVO createReqVO);

    /**
     * 更新投入品批次
     *
     * @param updateReqVO 更新信息
     */
    void updateInputsBatch(@Valid InputsBatchSaveReqVO updateReqVO);

    /**
     * 删除投入品批次
     *
     * @param id 编号
     */
    void deleteInputsBatch(Long id);

    /**
     * 获得投入品批次
     *
     * @param id 编号
     * @return 投入品批次
     */
    InputsBatchDO getInputsBatch(Long id);

    /**
     * 获得投入品批次列表
     *
     * @param ids 编号
     * @return 投入品批次列表
     */
    List<InputsBatchDO> getInputsBatchList(Collection<Long> ids);

    /**
     * 获得投入品批次分页
     *
     * @param pageReqVO 分页查询
     * @return 投入品批次分页
     */
    PageResult<InputsBatchDO> getInputsBatchPage(InputsBatchPageReqVO pageReqVO);

    /**
     * 获得投入品批次列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 投入品批次列表
     */
    List<InputsBatchDO> getInputsBatchList(InputsBatchExportReqVO exportReqVO);

    /**
     * 根据数量获得投入品批次编号
     * @param count
     * @return
     */
    List<String> getInputsBatchNumberList(Integer count);

    /**
     * 批量删除投入品批次
     * @param ids
     */
    void deleteBatch(List<Long> ids);

    /**
     * 根据投入品id获取所有投入品批次
     * @param inputsId
     * @return
     */
    List<String>getInputsBatchInputsBatchIdList(String inputsId);

    /**
     * 更新链状态
     * @param id id
     * @param state 状态
     */
    void updateChainStatus(Long id, Integer state);
}
