package cn.iocoder.yudao.module.product.service.inputsbuy;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbuy.InputsBuyDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 投入品采购 Service 接口
 *
 * @author 芋道源码
 */
public interface InputsBuyService {

    /**
     * 创建投入品采购
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInputsBuy(@Valid InputsBuyCreateReqVO createReqVO);

    /**
     * 更新投入品采购
     *
     * @param updateReqVO 更新信息
     */
    void updateInputsBuy(@Valid InputsBuyUpdateReqVO updateReqVO);

    /**
     * 删除投入品采购
     *
     * @param id 编号
     */
    void deleteInputsBuy(Long id);

    /**
     * 获得投入品采购
     *
     * @param id 编号
     * @return 投入品采购
     */
    InputsBuyRespVO getInputsBuy(Long id);

    /**
     * 获得投入品采购列表
     *
     * @param ids 编号
     * @return 投入品采购列表
     */
    List<InputsBuyDO> getInputsBuyList(Collection<Long> ids);

    /**
     * 获得投入品采购分页
     *
     * @param pageReqVO 分页查询
     * @return 投入品采购分页
     */
    PageResult<InputsBuyRespVO> getInputsBuyPage(InputsBuyPageReqVO pageReqVO);

    /**
     * 根据数量获得投入品批次编号
     * @param count
     * @return
     */
    List<String> getInputsBuyNumberList(Integer count);

    /**
     * 根据出库单单号获取投入品采购
     * @param outBoundId
     * @return
     */
    InputBuyForOutBoundRespVO getInputsBuyByOutbound(String outBoundId);

    /**
     * 上链
     * @param id id
     */
    void upChain(Long id);

    /**
     * 更新链状态
     * @param id id
     * @param state 状态
     */
    void updateChainStatus(Long id, Integer state);

}
