package cn.iocoder.yudao.module.product.service.inputsbuy;

import cn.hutool.core.bean.BeanUtil;
import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.basic.api.goodsinputrelational.GoodsInputRelationalApi;
import cn.iocoder.yudao.module.basic.api.goodsinputrelational.dto.RelationalRespDTO;
import cn.iocoder.yudao.module.basic.api.inputsinfo.InputsInfoApi;
import cn.iocoder.yudao.module.basic.api.inputsinfo.dto.InputsInfoRespDTO;
import cn.iocoder.yudao.module.basic.api.vendorinfo.VendorInfoApi;
import cn.iocoder.yudao.module.basic.api.vendorinfo.dto.VendorInfoRespDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.BlockCertificateApi;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.BlockCertificateRespDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.BlockCertificateUpChainReqDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.UpChainObjReqDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.UpdateStatusReqDTO;
import cn.iocoder.yudao.module.chain.enums.BatchTypeEnum;
import cn.iocoder.yudao.module.chain.enums.ChainStatusEnum;
import cn.iocoder.yudao.module.chain.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.flow.api.outbound.OutboundApi;
import cn.iocoder.yudao.module.flow.api.outbound.dto.OutboundGoodsDTO;
import cn.iocoder.yudao.module.flow.api.outbound.dto.OutboundRespDTO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.InputsBatchRespVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.InputsBatchSaveReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.*;
import cn.iocoder.yudao.module.product.convert.inputsbatch.InputsBatchConvert;
import cn.iocoder.yudao.module.product.convert.inputsbuy.InputsBuyConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbatch.InputsBatchDO;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbuy.InputsBuyDO;
import cn.iocoder.yudao.module.product.dal.mysql.inputsbatch.InputsBatchMapper;
import cn.iocoder.yudao.module.product.dal.mysql.inputsbuy.InputsBuyMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.seata.core.context.RootContext;
import io.seata.spring.annotation.GlobalTransactional;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 投入品采购 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
@Slf4j
public class InputsBuyServiceImpl implements InputsBuyService {

    @Resource
    private InputsBuyMapper inputsBuyMapper;
    @Resource
    private InputsBatchMapper inputsBatchMapper;
    @Resource
    private VendorInfoApi vendorInfoApi;
    @Resource
    private InputsInfoApi inputsInfoApi;
    @Resource
    private GoodsInputRelationalApi goodsInputRelationalApi;
    @Resource
    private OutboundApi outboundApi;
    @Resource
    private BlockCertificateApi blockCertificateApi;

    @Override
    @Transactional
    public Long createInputsBuy(InputsBuyCreateReqVO createReqVO) {
        // 检验供应商vendorId
        CommonResult<VendorInfoRespDTO> result = vendorInfoApi.getVendorInfoByVendorId(createReqVO.getVendorId());
        VendorInfoRespDTO VIResp = result.getData();
        if(result.isError() || VIResp == null) {
            throw ServiceExceptionUtil.exception(cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.VENDOR_INFO_NOT_EXISTS);
        }

        List<InputsBatchDO> inputsBatchs = InputsBatchConvert.INSTANCE.convertListForCreate(createReqVO.getInputsBatchs());
        Map<String, String> idAndNames = new HashMap<>();
        // 判断投入品是否存在
        this.verifiInputInfos(inputsBatchs, idAndNames, null, null, VIResp.getBelongBusinessId());

        inputsBatchs.forEach(inputsBatch -> {
            // inputInfoIds.add(inputsBatch.getInputsId());
            inputsBatch.setId(null);
            inputsBatch.setInputsBuyId(createReqVO.getInputsBuyId());
            inputsBatch.setInputsName(idAndNames.get(inputsBatch.getInputsId()));
            inputsBatch.setBlockState(BlockStateEnum.NOT_ON_CHAIN.getBlockState());
            inputsBatch.setBelongBusinessId(VIResp.getBelongBusinessId());
            inputsBatch.setBelongBusinessName(VIResp.getBelongBusinessName());
            if(inputsBatch.getInputsUnitPrice() != null) {
                // 重新计算总额
                inputsBatch.setTotalPrice(inputsBatch.getInputsUnitPrice().multiply(new BigDecimal(inputsBatch.getInputsNum())));
            } else {
                inputsBatch.setTotalPrice(new BigDecimal(0));
            }
        });

        // 包装需要插入的数据
        InputsBuyDO inputsBuy = InputsBuyConvert.INSTANCE.convert(createReqVO);
        // 下面的以数据库的为准
        inputsBuy.setVendorName(VIResp.getVendorName());
        inputsBuy.setBelongBusinessId(VIResp.getBelongBusinessId());
        inputsBuy.setBelongBusinessName(VIResp.getBelongBusinessName());

        inputsBuyMapper.insert(inputsBuy);

        inputsBatchMapper.insertBatch(inputsBatchs);
        // 返回
        return inputsBuy.getId();
    }

    /**
     * 更新投入品采购信息
     * （这里不考虑删除，删除功能在页面手动一个个删除，如果页面不传已有的数据，那么就当不变更处理）
     * @param updateReqVO 更新信息
     */
    @Override
    @Transactional
    public void updateInputsBuy(InputsBuyUpdateReqVO updateReqVO) {
        // 校验投入品采购主表信息
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(updateReqVO.getId());
        if(inputsBuy == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BUY_NOT_EXISTS);
        }

        List<InputsBatchDO> inputsBatchs = InputsBatchConvert.INSTANCE.convertListForCreate(updateReqVO.getInputsBatchs());
        Map<String, String> idAndNames = new HashMap<>();// 页面传过来的投入品键值对 <inputId, inputName>
        Map<String, InputsInfoRespDTO> inputsInfoMaps = new HashMap<>();// 页面传入的inputid集合
        List<Long> willUpdateIds = new ArrayList<>();// 将要更新的投入品批次主键id

        // 判断投入品是否存在
        this.verifiInputInfos(inputsBatchs, idAndNames, inputsInfoMaps, willUpdateIds, inputsBuy.getBelongBusinessId());

        // 验证将要更新的批次
        Map<Long, InputsBatchDO> inputBatchMaps = new HashMap<>();// 投入品批次键值对
        if(willUpdateIds.size() > 0) {
            // 查询需要更新的投入品批次
            List<InputsBatchDO> inputBatchDOs = inputsBatchMapper.selectList(new LambdaQueryWrapperX<InputsBatchDO>()
                    .in(InputsBatchDO::getId, willUpdateIds)
                    .eq(InputsBatchDO::getInputsBuyId, inputsBuy.getInputsBuyId()));
            if (inputBatchDOs.size() != willUpdateIds.size()) { // 存在已删除或者不是改投入品采购单的批次信息
                throw ServiceExceptionUtil.exception(ErrorCodeConstants.EXIST_DELETED_INPUT_BATCH);
            }
//            if(!inputBatchDOs.get(0).getBlockState().equals(BlockStateEnum.NOT_ON_CHAIN.getBlockState())) {
//                throw ServiceExceptionUtil.exception(ErrorCodeConstants.ONLY_NOT_ON_CHAIN_INPUTS_BUY_CAN_BE_EDIT);
//            }

            inputBatchDOs.forEach(item -> inputBatchMaps.put(item.getId(), item));
        }

        List<InputsBatchDO> addList = new ArrayList<>(); // 需要新增的批次集合
        List<InputsBatchDO> updateList = new ArrayList<>(); // 需要新增的批次集合
        // List<InputsBatchDO> deleteList = new ArrayList<>(); // 需要删除的批次集合

        for(InputsBatchSaveReqVO item: updateReqVO.getInputsBatchs()) {
            if(item.getId() != null) {
                // 更新流程
                InputsBatchDO inputsBatch = inputBatchMaps.get(item.getId());
                // 只更新可更新的字段
                inputsBatch.setProductData(item.getProductData());
                inputsBatch.setInputsNum(item.getInputsNum());
                inputsBatch.setInputsWeight(item.getInputsWeight());
                inputsBatch.setInputsUnitPrice(item.getInputsUnitPrice());
                inputsBatch.setRemarks(item.getRemarks());
                if(inputsBatch.getInputsUnitPrice() != null) {
                    // 重新计算总额
                    inputsBatch.setTotalPrice(inputsBatch.getInputsUnitPrice().multiply(new BigDecimal(inputsBatch.getInputsNum())));
                } else {
                    inputsBatch.setTotalPrice(new BigDecimal(0));
                }
                updateList.add(inputsBatch);
            } else {
                InputsBatchDO inputsBatch = InputsBatchConvert.INSTANCE.convert(item);
                // 新增 这里都以数据库的数据为准
                inputsBatch.setInputsBuyId(inputsBuy.getInputsBuyId());
                inputsBatch.setBlockState(BlockStateEnum.NOT_ON_CHAIN.getBlockState());
                inputsBatch.setInputsName(inputsInfoMaps.get(item.getInputsId()).getInputsName());
                inputsBatch.setBelongBusinessId(inputsBuy.getBelongBusinessId());
                inputsBatch.setBelongBusinessName(inputsBuy.getBelongBusinessName());
                inputsBatch.setInputsUnit(inputsInfoMaps.get(item.getInputsId()).getInputsUnit());
                if(inputsBatch.getInputsUnitPrice() != null) {
                    // 重新计算总额
                    inputsBatch.setTotalPrice(inputsBatch.getInputsUnitPrice().multiply(new BigDecimal(inputsBatch.getInputsNum())));
                } else {
                    inputsBatch.setTotalPrice(new BigDecimal(0));
                }
                addList.add(inputsBatch);
            }
        }

        // 更新主单
        InputsBuyDO updateInputBuy = new InputsBuyDO();
        updateInputBuy.setId(updateReqVO.getId());
        updateInputBuy.setRelateOutboundId(updateReqVO.getRelateOutboundId());
        updateInputBuy.setInputsBuyTime(updateReqVO.getInputsBuyTime());

        if(!addList.isEmpty()) {
            inputsBatchMapper.insertBatch(addList);
        }

        if(!updateList.isEmpty()) {
            inputsBatchMapper.updateBatch(updateList, Integer.MAX_VALUE);
        }

        inputsBuyMapper.updateById(updateInputBuy);
    }

    /**
     *
     * @param inputsBatchs  页面传入的InputsBatchDO集合
     * @param idAndNames   id和名字的键值对，
     * @param inputsInfoMaps  投入品编号 <key:投入品编号, value:投入品名称>
     * @param willUpdateIds 将要更新的数据的批次id
     */
    private void verifiInputInfos(List<InputsBatchDO> inputsBatchs, Map<String, String> idAndNames, Map<String, InputsInfoRespDTO> inputsInfoMaps,
            List<Long> willUpdateIds, Long businessId) {
        Set<String> inputInfoIdsSet = new HashSet<>();
        Set<Long> willUpdateIdsSet = new HashSet<>();
        inputsBatchs.forEach(inputsBatch -> {
            inputInfoIdsSet.add(inputsBatch.getInputsId());
            if(inputsBatch.getId() != null) {
                willUpdateIdsSet.add(inputsBatch.getId());
                if(willUpdateIds != null) {
                    willUpdateIds.add(inputsBatch.getId());
                }
            }
        });

        // 存在相同的投入品编号
        if(inputInfoIdsSet.size() != inputsBatchs.size()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.CAN_NOT_ADD_THE_SAME_INPUT_INFO);
        }
        // 存在相同的批次编号
        if(willUpdateIds != null && willUpdateIdsSet.size() != willUpdateIds.size()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.EXIST_THE_SAME_INPUT_BATCH);
        }

        // 这边开始校验投入品是否存在
        List<String> inputInfoIds = new ArrayList<>(inputInfoIdsSet);
        List<InputsInfoRespDTO> inputsInfoRespDTO = inputsInfoApi.getInputsInfoListByInputId(inputInfoIds).getData();
        if(inputsInfoRespDTO.isEmpty() || inputsInfoRespDTO.size() != inputInfoIds.size()) {
            throw ServiceExceptionUtil.exception(cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.INPUTS_INFO_NOT_EXISTS);
        }

        inputsInfoRespDTO.forEach(item -> {
            if(!item.getBelongBusinessId().equals(businessId)) {
                throw ServiceExceptionUtil.exception(cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.INPUTS_INFO_NOT_EXISTS);
            }
            idAndNames.put(item.getInputsId(), item.getInputsName());
            if(inputsInfoMaps != null) {
                inputsInfoMaps.put(item.getInputsId(), item);
            }
        });
    }

    @Override
    public void deleteInputsBuy(Long id) {
        // 校验存在
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(id);
        if(inputsBuy == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BUY_NOT_EXISTS);
        }
        // 删除
        inputsBuyMapper.deleteById(id);
    }

    @Override
    public InputsBuyRespVO getInputsBuy(Long id) {
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(id);
        if(inputsBuy == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BUY_NOT_EXISTS);
        }
        InputsBuyRespVO inputsBuyRespVO = InputsBuyConvert.INSTANCE.convert(inputsBuy);

        List<InputsBatchDO> list = inputsBatchMapper.selectList(InputsBatchDO::getInputsBuyId, inputsBuyRespVO.getInputsBuyId());
        List<InputsBatchRespVO> inputsBatchVOs = InputsBatchConvert.INSTANCE.convertList(list);
        inputsBuyRespVO.setInputsBatchs(inputsBatchVOs);
        return inputsBuyRespVO;
    }

    @Override
    public List<InputsBuyDO> getInputsBuyList(Collection<Long> ids) {
        return inputsBuyMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<InputsBuyRespVO> getInputsBuyPage(InputsBuyPageReqVO pageReqVO) {
        Page<InputsBuyRespVO> page = new Page<>(pageReqVO.getPageNo(), pageReqVO.getPageSize());
        List<InputsBuyRespVO> result = inputsBuyMapper.selectInputsBuyPage(page, pageReqVO);
        // 按照已生成时间的倒序排序
        result.sort(Comparator.comparing(InputsBuyRespVO::getCreateTime).reversed());
        return new PageResult<>(result, page.getTotal());
    }

    @Override
    public List<String> getInputsBuyNumberList(Integer count) {
        List<String> results = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            results.add(CodeEnum.INPUTS_BUY_CODE.getCode());
        }
        return results;
    }

    @Override
    public InputBuyForOutBoundRespVO getInputsBuyByOutbound(String outBoundId) {
        // 根据出库单号查询出库单信息
        CommonResult<OutboundRespDTO> outResp = outboundApi.getOutboundByOutBoundId(outBoundId);
        if(outResp.isError()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.OUTBOUND_NOT_EXISTS);
        }
//        if(!outResp.getData().getBlockState().equals(BlockStateEnum.ACTIVATION.getBlockState())) {
//            throw ServiceExceptionUtil.exception(ErrorCodeConstants.OUTBOUND_NOT_ACTIVATION);
//        }

        OutboundRespDTO outboundRespDTO = outResp.getData();
        List<String> goodsIds = outboundRespDTO.getOutboundGoodsList().stream().map(OutboundGoodsDTO::getGoodsId).collect(Collectors.toList());
        // 根据商品编号查询投入品信息
        // CommonResult<List<GoodsInputRelationalRespDTO>> inputInfoResult = goodsInputRelationalApi.getInputInfoByGoodsId(goodsIds);
        CommonResult<RelationalRespDTO> relaResult = goodsInputRelationalApi.getInputInfoByGoodsId(goodsIds);
        if(relaResult.isError()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.GOODS_INFO_NOT_EXISTS);
        }

        InputBuyForOutBoundRespVO result = new InputBuyForOutBoundRespVO();
        if(relaResult.getData().getInputsInfo() == null || relaResult.getData().getInputsInfo().isEmpty()) {
            // 说明出库单里面有 没和投入品关联的商品
            result.setNeedRelaInputInfo(Boolean.TRUE);
            result.setGoodsInfos(InputsBuyConvert.INSTANCE.convertToGoodsInfoList(relaResult.getData().getGoodsInfos()));
            result.setBelongBusinessId(outResp.getData().getBelongBusinessId());
            // 此时返回没有关联的商品
            return result;
        }

        // 返回投入品信息
        // List<InputsBatchRespVO> inputsBatchVOs = InputsBatchConvert.INSTANCE.convertOfDTOList(relaResult.getData().getInputsInfo());

        List<InputsBatchRespVO> inputsBatchVOs = relaResult.getData().getInputsInfo().stream().map(r -> {
            InputsBatchRespVO InputsBatchRespVO = new InputsBatchRespVO();
            BeanUtil.copyProperties(r, InputsBatchRespVO);
            InputsBatchRespVO.setInputsBatchId(CodeEnum.INPUTS_BATCH_CODE.getCode());
            // 设置商品数量
            // InputsBatchRespVO.setInputsNum(outGoodsMap.get(r.getGoodsId()).getGoodsWeight());
            return InputsBatchRespVO;
        }).collect(Collectors.toList());

        result.setNeedRelaInputInfo(Boolean.FALSE);
        result.setInputsBuyTime(outboundRespDTO.getOutBoundTime());
        result.setBelongBusinessId(outboundRespDTO.getBelongBusinessId());
        result.setBelongBusinessName(outboundRespDTO.getBelongBusinessName());
        result.setVendorId(outboundRespDTO.getVendorId());
        result.setVendorName(outboundRespDTO.getVendorName());
        result.setInputsBuyId(CodeEnum.INPUTS_BUY_CODE.getCode());
        result.setRelateOutboundId(outBoundId);
        result.setInputsBatchs(inputsBatchVOs);
        return result;
    }

    @Override
    @GlobalTransactional(rollbackFor = Exception.class)
    public void upChain(Long id) {
        log.info("[reduceStock] 当前 XID: {}", RootContext.getXID());
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(id);
        if(inputsBuy == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BUY_NOT_EXISTS);
        }
        if(!inputsBuy.getBlockState().equals(BlockStateEnum.NOT_ON_CHAIN.getBlockState())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.ONLY_NOT_ON_CHAIN_INPUT_BUY_CAN_DO_UP_CHAIN);
        }
        // 查询所有批次
        List<InputsBatchDO> inputsBatchs = inputsBatchMapper.selectList(new LambdaQueryWrapperX<InputsBatchDO>().eq(InputsBatchDO::getInputsBuyId, inputsBuy.getInputsBuyId()));
        // 封装批次
        BlockCertificateUpChainReqDTO upChainBatchReqDTO = new BlockCertificateUpChainReqDTO();
        List<UpChainObjReqDTO> operateBatchs = inputsBatchs.stream().map(batch -> {
            UpChainObjReqDTO upChainObjReqDTO = new UpChainObjReqDTO();
            upChainObjReqDTO.setBatchId(batch.getInputsBatchId());
            upChainObjReqDTO.setOperateObjId(batch.getId());
            return upChainObjReqDTO;
        }).collect(Collectors.toList());
        upChainBatchReqDTO.setOperateType(OperateTypeEnum.BATCH.getType());
        upChainBatchReqDTO.setOperateName(OperateTypeEnum.BATCH.getName());
        upChainBatchReqDTO.setBatchType(BatchTypeEnum.INPUT_BATCH.getType());
        upChainBatchReqDTO.setOperateObjs(operateBatchs);
        upChainBatchReqDTO.setBelongBusinessId(inputsBuy.getBelongBusinessId());
        upChainBatchReqDTO.setBelongBusinessName(inputsBuy.getBelongBusinessName());

        // 封装投入品
        List<UpChainObjReqDTO> operateHarvests = new ArrayList<>();
        UpChainObjReqDTO upChainObjReqDTO = new UpChainObjReqDTO();
        upChainObjReqDTO.setOperateObjId(id);
        operateHarvests.add(upChainObjReqDTO);

        BlockCertificateUpChainReqDTO upChainReqDTO = new BlockCertificateUpChainReqDTO();
        upChainReqDTO.setOperateObjs(operateHarvests);
        upChainReqDTO.setOperateType(OperateTypeEnum.INPUT_BUY.getType());
        upChainReqDTO.setOperateName(OperateTypeEnum.INPUT_BUY.getName());
        upChainReqDTO.setBelongBusinessId(inputsBuy.getBelongBusinessId());
        upChainReqDTO.setBelongBusinessName(inputsBuy.getBelongBusinessName());

        // 远程调用
        List<BlockCertificateRespDTO> inputBuyResults = blockCertificateApi.createAndUpChainBatch(upChainReqDTO).getData();
        List<BlockCertificateRespDTO> batchResults = blockCertificateApi.createAndUpChainBatch(upChainBatchReqDTO).getData();

        if(batchResults == null || inputBuyResults == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.CREATE_BLOCK_CERTIFICATE_FAIL);
        }
        Map<String, BlockCertificateRespDTO> batchBCMaps = new HashMap<>();
        batchResults.forEach(item -> batchBCMaps.put(item.getBatchId(), item));
        // 将要更新的类
        InputsBuyDO inputsBuyDO = new InputsBuyDO();
        inputsBuyDO.setId(id);
        inputsBuyDO.setBlockChainBlockIdentity(inputBuyResults.get(0).getChainBlockIdentity());
        inputsBuyDO.setBlockChainId(inputBuyResults.get(0).getChainOperateId());
        inputsBuyDO.setBlockState(BlockStateEnum.ACTIVATION.getBlockState());// 改为激活状态

        List<InputsBatchDO> inputsBatchDOS = new ArrayList<>();
        //循环所有批次
        inputsBatchs.forEach(item -> {
            InputsBatchDO inputsBatchDO = new InputsBatchDO();
            BeanUtil.copyProperties(item, inputsBatchDO);
            inputsBatchDO.setBlockChainBlockIdentity(batchBCMaps.get(item.getInputsBatchId()).getChainBlockIdentity());
            inputsBatchDO.setBlockChainId(batchBCMaps.get(item.getInputsBatchId()).getChainOperateId());
            inputsBatchDO.setBlockState(BlockStateEnum.ACTIVATION.getBlockState());// 修改主单的激活状态时， 子单一律改为激活
            inputsBatchDOS.add(inputsBatchDO);
        });

        inputsBuyMapper.updateById(inputsBuyDO);
        inputsBatchMapper.updateBatch(inputsBatchDOS, Integer.MAX_VALUE);
    }

    @Override
    @GlobalTransactional(rollbackFor = Exception.class)
    public void updateChainStatus(Long id, Integer state) {
        log.info("[reduceStock] 当前 XID: {}", RootContext.getXID());
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(id);
        if(inputsBuy == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BUY_NOT_EXISTS);
        }
        if(inputsBuy.getBlockState().equals(BlockStateEnum.NOT_ON_CHAIN.getBlockState())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.ONLY_ON_CHAIN_INPUT_BUY_CAN_DO_UPDATE_CHAIN_STATUS);
        }
        // 查询所有批次
        List<InputsBatchDO> inputsBatchs = inputsBatchMapper.selectList(new LambdaQueryWrapperX<InputsBatchDO>().eq(InputsBatchDO::getInputsBuyId, inputsBuy.getInputsBuyId()));

        Integer blockState = state.equals(BlockStateEnum.ACTIVATION.getBlockState()) ? BlockStateEnum.ACTIVATION.getBlockState() : BlockStateEnum.FREEZE.getBlockState();
        Integer chainStatus = state.equals(BlockStateEnum.ACTIVATION.getBlockState()) ? ChainStatusEnum.ACTIVE.getStatus() : ChainStatusEnum.FREEZE.getStatus();

        // 修改主单的链状态时，将同步修改子单的链状态
        List<UpdateStatusReqDTO> batchsReqDTOS = new ArrayList<>();
        UpdateStatusReqDTO harvestReqDTO = new UpdateStatusReqDTO();
        harvestReqDTO.setChainOperateId(inputsBuy.getBlockChainId());
        harvestReqDTO.setChainStatus(chainStatus);
        batchsReqDTOS.add(harvestReqDTO);
        inputsBatchs.forEach(batch -> {
            UpdateStatusReqDTO batchReqDTO = new UpdateStatusReqDTO();
            batchReqDTO.setChainOperateId(batch.getBlockChainId());
            batchReqDTO.setChainStatus(chainStatus);
            batchsReqDTOS.add(batchReqDTO);
        });

        if(blockCertificateApi.updateStatusBatch(batchsReqDTOS).isError()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.UPDATE_CREATE_BLOCK_CERTIFICATE_FAIL);
        }

        InputsBuyDO inputsBuyDO = new InputsBuyDO();
        inputsBuyDO.setId(id);
        inputsBuyDO.setBlockState(blockState);

        inputsBatchs.forEach(batch -> batch.setBlockState(blockState));

        inputsBuyMapper.updateById(inputsBuyDO);
        inputsBatchMapper.updateBatch(inputsBatchs, Integer.MAX_VALUE);
    }

}
