package cn.iocoder.yudao.module.product.service.landblockstate;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStatePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.landblockstate.LandBlockStateDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 地块状态 Service 接口
 *
 * @author 芋道源码
 */
public interface LandBlockStateService {

    /**
     * 创建地块状态
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLandBlockState(@Valid LandBlockStateCreateReqVO createReqVO);

    /**
     * 更新地块状态
     *
     * @param updateReqVO 更新信息
     */
    void updateLandBlockState(@Valid LandBlockStateUpdateReqVO updateReqVO);

    /**
     * 删除地块状态
     *
     * @param id 编号
     */
    void deleteLandBlockState(Long id);

    /**
     * 获得地块状态
     *
     * @param id 编号
     * @return 地块状态
     */
    LandBlockStateDO getLandBlockState(Long id);

    /**
     * 获得地块状态列表
     *
     * @param ids 编号
     * @return 地块状态列表
     */
    List<LandBlockStateDO> getLandBlockStateList(Collection<Long> ids);

    /**
     * 获得地块状态分页
     *
     * @param pageReqVO 分页查询
     * @return 地块状态分页
     */
    PageResult<LandBlockStateDO> getLandBlockStatePage(LandBlockStatePageReqVO pageReqVO);

    /**
     * 根据商户ID获取所有地块状态
     * @param id
     * @return
     */
    List<LandBlockStateDO> getByBusinessInfoId(Long id);

}
