package cn.iocoder.yudao.module.product.service.landblockstate;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.basic.api.businessinfo.BusinessInfoApi;
import cn.iocoder.yudao.module.basic.api.redis.dto.BusinessInfoDTO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStatePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landblockstate.vo.LandBlockStateUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landblockstate.LandBlockStateConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landblockstate.LandBlockStateDO;
import cn.iocoder.yudao.module.product.dal.mysql.landblockstate.LandBlockStateMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_BLOCK_STATE_ALREADY_EXIST;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_BLOCK_STATE_NOT_EXISTS;

/**
 * 地块状态 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandBlockStateServiceImpl implements LandBlockStateService {

    @Resource
    private LandBlockStateMapper landBlockStateMapper;
    @Resource
    private BusinessInfoApi businessInfoApi;

    @Override
    public Long createLandBlockState(LandBlockStateCreateReqVO createReqVO) {
        //校验商户昵称的地块状态名称是否重复
        validateLandStateNameExists(createReqVO.getLandStateName(),createReqVO.getBelongBusinessId());
        //校验商户
        BusinessInfoDTO businessInfo = businessInfoApi.getBusinessInfoById(createReqVO.getBelongBusinessId()).getData();
        if(businessInfo == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.BUSINESS_INFO_NOT_EXISTS);
        }
        // 插入
        LandBlockStateDO landBlockState = LandBlockStateConvert.INSTANCE.convert(createReqVO);
        landBlockState.setBelongBusinessName(businessInfo.getBusinessName());
        landBlockStateMapper.insert(landBlockState);
        // 返回
        return landBlockState.getId();
    }

    @Override
    public void updateLandBlockState(LandBlockStateUpdateReqVO updateReqVO) {
        //校验是否存在
        LandBlockStateDO landBlockStateInfo = landBlockStateMapper.selectById(updateReqVO.getId());
        if (landBlockStateInfo == null){
            throw exception(LAND_BLOCK_STATE_NOT_EXISTS);
        }
        // 检查地块状态是否发生变化
        if (!Objects.equals(landBlockStateInfo.getLandStateName(), updateReqVO.getLandStateName())) {
            //校验商户昵称的地块状态名称是否重复
            validateLandStateNameExists(updateReqVO.getLandStateName(),updateReqVO.getBelongBusinessId());
        }
        // 更新
        LandBlockStateDO updateObj = LandBlockStateConvert.INSTANCE.convert(updateReqVO);
        landBlockStateMapper.updateById(updateObj);
    }

    @Override
    public void deleteLandBlockState(Long id) {
        // 校验存在
        validateLandBlockStateExists(id);
        // 删除
        landBlockStateMapper.deleteById(id);
    }

    private void validateLandBlockStateExists(Long id) {
        if (landBlockStateMapper.selectById(id) == null) {
            throw exception(LAND_BLOCK_STATE_NOT_EXISTS);
        }
    }

    @Override
    public LandBlockStateDO getLandBlockState(Long id) {
        return landBlockStateMapper.selectById(id);
    }

    @Override
    public List<LandBlockStateDO> getLandBlockStateList(Collection<Long> ids) {
        return landBlockStateMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<LandBlockStateDO> getLandBlockStatePage(LandBlockStatePageReqVO pageReqVO) {
        return landBlockStateMapper.selectPage(pageReqVO);
    }

    /**
     * 根据商户ID获取所有地块状态
     *
     * @param id
     * @return
     */
    @Override
    public List<LandBlockStateDO> getByBusinessInfoId(Long id) {
        return landBlockStateMapper.selectList(new LambdaQueryWrapperX<LandBlockStateDO>().eq(LandBlockStateDO::getBelongBusinessId, id));
    }

    private void validateLandStateNameExists(String landStateName,Long belongBusinessId) {
        QueryWrapper<LandBlockStateDO> wrapper = new QueryWrapper<>();
        wrapper.eq("land_state_name", landStateName)
                .eq("belong_business_id",belongBusinessId);
        if (landBlockStateMapper.selectOne(wrapper) != null) {
            throw exception(LAND_BLOCK_STATE_ALREADY_EXIST);
        }
    }

}
