package cn.iocoder.yudao.module.product.service.landinfo;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfo.LandInfoDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 农事土地信息 Service 接口
 *
 * @author 芋道源码
 */
public interface LandInfoService {

    /**
     * 创建农事土地信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLandInfo(@Valid LandInfoCreateReqVO createReqVO);

    /**
     * 更新农事土地信息
     *
     * @param updateReqVO 更新信息
     */
    void updateLandInfo(@Valid LandInfoUpdateReqVO updateReqVO);

    /**
     * 删除农事土地信息
     *
     * @param id 编号
     */
    void deleteLandInfo(Long id);

    /**
     * 获得农事土地信息
     *
     * @param id 编号
     * @return 农事土地信息
     */
    LandInfoDO getLandInfo(Long id);

    /**
     * 根据地块编码获取地块信息
     * @param landId 地块编码
     * @return 地块信息
     */
    LandInfoDO getLandInfoByLandId(String landId);

    /**
     * 获得农事土地信息列表
     *
     * @param ids 编号
     * @return 农事土地信息列表
     */
    List<LandInfoDO> getLandInfoList(Collection<Long> ids);

    /**
     * 获得农事土地信息分页
     *
     * @param pageReqVO 分页查询
     * @return 农事土地信息分页
     */
    PageResult<LandInfoDO> getLandInfoPage(LandInfoPageReqVO pageReqVO);

    /**
     * 获取所有地块昵称
     * @return
     */
    List<LandInfoDO> getLandInfoLandNameList();

    /**
     * 根据地块id获取所有该地块的商户名称
     * @param
     * @return
     */
    List<LandInfoDO> getLandInfoByBelongBusinessId(Long belongBusinessId);

}
