package cn.iocoder.yudao.module.product.service.landinfotype;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfotype.LandInfoTypeDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 地块类型 Service 接口
 *
 * @author 芋道源码
 */
public interface LandInfoTypeService {

    /**
     * 创建地块类型
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLandInfoType(@Valid LandInfoTypeCreateReqVO createReqVO);

    /**
     * 更新地块类型
     *
     * @param updateReqVO 更新信息
     */
    void updateLandInfoType(@Valid LandInfoTypeUpdateReqVO updateReqVO);

    /**
     * 删除地块类型
     *
     * @param id 编号
     */
    void deleteLandInfoType(Long id);

    /**
     * 获得地块类型
     *
     * @param id 编号
     * @return 地块类型
     */
    LandInfoTypeDO getLandInfoType(Long id);

    /**
     * 获得地块类型列表
     *
     * @param ids 编号
     * @return 地块类型列表
     */
    List<LandInfoTypeDO> getLandInfoTypeList(Collection<Long> ids);

    /**
     * 获得地块类型分页
     *
     * @param pageReqVO 分页查询
     * @return 地块类型分页
     */
    PageResult<LandInfoTypeDO> getLandInfoTypePage(LandInfoTypePageReqVO pageReqVO);

    /**
     * 根据商户ID获取所有地块类型
     * @param id
     * @return
     */
    List<LandInfoTypeDO> getByBusinessInfoId(Long id);

}
