package cn.iocoder.yudao.module.product.service.landinfotype;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.basic.api.businessinfo.BusinessInfoApi;
import cn.iocoder.yudao.module.basic.api.redis.dto.BusinessInfoDTO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfotype.vo.LandInfoTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landinfotype.LandInfoTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfotype.LandInfoTypeDO;
import cn.iocoder.yudao.module.product.dal.mysql.landinfotype.LandInfoTypeMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_INFO_TYPE_ALREADY_EXIST;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_INFO_TYPE_NOT_EXISTS;

/**
 * 地块类型 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandInfoTypeServiceImpl implements LandInfoTypeService {

    @Resource
    private LandInfoTypeMapper landInfoTypeMapper;

    @Resource
    private BusinessInfoApi businessInfoApi;

    @Override
    public Long createLandInfoType(LandInfoTypeCreateReqVO createReqVO) {
        //校验商户昵称的地块类型名称是否重复
        validateLandTypeNameExists(createReqVO.getLandTypeName(),createReqVO.getBelongBusinessId());
        // 校验商户
        BusinessInfoDTO businessInfo = businessInfoApi.getBusinessInfoById(createReqVO.getBelongBusinessId()).getData();
        if(businessInfo == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.BUSINESS_INFO_NOT_EXISTS);
        }
        // 插入
        LandInfoTypeDO landInfoType = LandInfoTypeConvert.INSTANCE.convert(createReqVO);
        landInfoType.setBelongBusinessName(businessInfo.getBusinessName());
        landInfoTypeMapper.insert(landInfoType);
        // 返回
        return landInfoType.getId();
    }

    @Override
    public void updateLandInfoType(LandInfoTypeUpdateReqVO updateReqVO) {
        //校验是否存在
        LandInfoTypeDO landInfoTypeInfo = landInfoTypeMapper.selectById(updateReqVO.getId());
        if (landInfoTypeInfo == null){
            throw exception(LAND_INFO_TYPE_NOT_EXISTS);
        }
        // 检查地块类型是否发生变化
        if (!Objects.equals(landInfoTypeInfo.getLandTypeName(), updateReqVO.getLandTypeName())) {
            //校验商户昵称的地块类型名称是否重复
            validateLandTypeNameExists(updateReqVO.getLandTypeName(),updateReqVO.getBelongBusinessId());
        }
        // 更新
        LandInfoTypeDO updateObj = LandInfoTypeConvert.INSTANCE.convert(updateReqVO);
        landInfoTypeMapper.updateById(updateObj);
    }

    @Override
    public void deleteLandInfoType(Long id) {
        // 校验存在
        validateLandInfoTypeExists(id);
        // 删除
        landInfoTypeMapper.deleteById(id);
    }

    private void validateLandInfoTypeExists(Long id) {
        if (landInfoTypeMapper.selectById(id) == null) {
            throw exception(LAND_INFO_TYPE_NOT_EXISTS);
        }
    }

    @Override
    public LandInfoTypeDO getLandInfoType(Long id) {
        return landInfoTypeMapper.selectById(id);
    }

    @Override
    public List<LandInfoTypeDO> getLandInfoTypeList(Collection<Long> ids) {
        return landInfoTypeMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<LandInfoTypeDO> getLandInfoTypePage(LandInfoTypePageReqVO pageReqVO) {
        return landInfoTypeMapper.selectPage(pageReqVO);
    }

    @Override
    public List<LandInfoTypeDO> getByBusinessInfoId(Long id) {
        return landInfoTypeMapper.selectList(new LambdaQueryWrapperX<LandInfoTypeDO>().eq(LandInfoTypeDO::getBelongBusinessId, id));
    }

    private void validateLandTypeNameExists(String landTypeName,Long belongBusinessId) {
        QueryWrapper<LandInfoTypeDO> wrapper = new QueryWrapper<>();
        wrapper.eq("land_type_name", landTypeName)
                .eq("belong_business_id",belongBusinessId);
        if (landInfoTypeMapper.selectOne(wrapper) != null) {
            throw exception(LAND_INFO_TYPE_ALREADY_EXIST);
        }
    }

}
