package cn.iocoder.yudao.module.product.service.landjob;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.*;
import cn.iocoder.yudao.module.product.controller.app.landjob.vo.FileUploadReqVO;
import cn.iocoder.yudao.module.product.controller.app.landjob.vo.LandJobReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.landjob.LandJobDO;

import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

/**
 * 种植/养殖/畜牧作业 Service 接口
 *
 * @author 芋道源码
 */
public interface LandJobService {

    /**
     * 创建种植/养殖/畜牧作业
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLandJob(@Valid LandJobCreateReqVO createReqVO);

    /**
     * 更新种植/养殖/畜牧作业
     *
     * @param updateReqVO 更新信息
     */
    void updateLandJob(@Valid LandJobUpdateReqVO updateReqVO);

    /**
     * 删除种植/养殖/畜牧作业
     *
     * @param id 编号
     */
    void deleteLandJob(Long id);

    /**
     * 获得种植/养殖/畜牧作业
     *
     * @param id 编号
     * @return 种植/养殖/畜牧作业
     */
    LandJobRespVO getLandJob(Long id);

    /**
     * 获得种植/养殖/畜牧作业列表
     *
     * @param ids 编号
     * @return 种植/养殖/畜牧作业列表
     */
    List<LandJobDO> getLandJobList(Collection<Long> ids);

    /**
     * 获得种植/养殖/畜牧作业分页
     *
     * @param pageReqVO 分页查询
     * @return 种植/养殖/畜牧作业分页
     */
    PageResult<LandJobDO> getLandJobPage(LandJobPageReqVO pageReqVO);

    /**
     * 获得种植/养殖/畜牧作业列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 种植/养殖/畜牧作业列表
     */
    List<LandJobDO> getLandJobList(LandJobExportReqVO exportReqVO);

    /**
     * 上链
     *
     * @param id id
     */
    void upChain(Long id);

    /**
     * 更新链状态
     *
     * @param id    id
     * @param state 状态
     */
    void updateChainStatus(Long id, Integer state);

    /**
     * 查询同类型的农事作业
     * @param landJobReqVO 入参
     * @return 农事作业
     */
    List<LandJobRespVO> getSameTypeJobById(LandJobReqVO landJobReqVO);

    /**
     * 新增农事作业
     * @param createOfAppReqVO 入参
     * @return ID
     */
    Long createLandJobByApp(LandJobCreateOfAppReqVO createOfAppReqVO);

    /**
     * 上传文件
     * @param uploadReqVO 入参
     * @return 文件名
     * @throws IOException 异常
     */
    String upload(FileUploadReqVO uploadReqVO) throws IOException;
}
