package cn.iocoder.yudao.module.product.service.landjob;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.IoUtil;
import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.tenant.core.context.TenantContextHolder;
import cn.iocoder.yudao.framework.tenant.core.util.TenantUtils;
import cn.iocoder.yudao.module.basic.api.businessinfo.BusinessInfoApi;
import cn.iocoder.yudao.module.basic.api.redis.dto.BusinessInfoDTO;
import cn.iocoder.yudao.module.basic.api.workersinfo.WorkersinfoApi;
import cn.iocoder.yudao.module.basic.api.workersinfo.dto.WorkersInfoRespDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.BlockCertificateApi;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.BlockCertificateRespDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.BlockCertificateUpChainReqDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.UpChainObjReqDTO;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.UpdateStatusReqDTO;
import cn.iocoder.yudao.module.chain.enums.ChainStatusEnum;
import cn.iocoder.yudao.module.chain.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.*;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsSaveReqVO;
import cn.iocoder.yudao.module.product.controller.app.landjob.vo.FileUploadReqVO;
import cn.iocoder.yudao.module.product.controller.app.landjob.vo.LandJobReqVO;
import cn.iocoder.yudao.module.product.convert.landjob.LandJobConvert;
import cn.iocoder.yudao.module.product.convert.landjobinputs.LandJobInputsConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfo.LandInfoDO;
import cn.iocoder.yudao.module.product.dal.dataobject.landjob.LandJobDO;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobinputs.LandJobInputsDO;
import cn.iocoder.yudao.module.product.dal.mysql.landinfo.LandInfoMapper;
import cn.iocoder.yudao.module.product.dal.mysql.landjob.LandJobMapper;
import cn.iocoder.yudao.module.product.dal.mysql.landjobinputs.LandJobInputsMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_INFO_NOT_EXISTS;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_JOB_NOT_EXISTS;

/**
 * 种植/养殖/畜牧作业 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandJobServiceImpl implements LandJobService {

    @Resource
    private LandJobMapper landJobMapper;

    @Resource
    private WorkersinfoApi workersinfoApi;

    @Resource
    private BusinessInfoApi businessInfoApi;

    @Resource
    private FileApi fileApi;

    @Resource
    private LandJobInputsMapper landJobInputsMapper;

    @Resource
    private LandInfoMapper landInfoMapper;

    @Resource
    private BlockCertificateApi blockCertificateApi;

    /**
     * 创建种植/养殖/畜牧作业
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    @Override
    @Transactional
    public Long createLandJob(LandJobCreateReqVO createReqVO) {
        //校验员工是否存在，如果不存在就抛出异常
        CommonResult<WorkersInfoRespDTO> workersInfo = workersinfoApi.getWorkersInfo(createReqVO.getJobWorkerId());
        WorkersInfoRespDTO dto = workersInfo.getData();
        if (workersInfo.isError() || dto == null){
            throw exception(cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.WORKERS_INFO_NOT_EXISTS);
        }

        // 插入
        LandJobDO landJob = LandJobConvert.INSTANCE.convert(createReqVO);
        //下面的数据参照数据库为准
        landJob.setBelongBusinessId(dto.getBelongBusinessId());
        landJob.setBelongBusinessName(dto.getBelongBusinessName());
        landJob.setBlockState(BlockStateEnum.NOT_ON_CHAIN.getBlockState());
        landJob.setJobWorker(dto.getWorkersName());
        landJob.setLandJobId(CodeEnum.LAND_JOB_CODE.getCode());

        //将批次信息转换成对应的DO对象，并进行存在性校验
        List<LandJobInputsDO> landJobInputsDOS = LandJobInputsConvert.INSTANCE.convertListCreate(createReqVO.getLandJobIds());
        landJobInputsDOS.forEach(landJobInputsDO -> {
            landJobInputsDO.setId(null);
            landJobInputsDO.setLandJobId(landJob.getLandJobId());
            landJobInputsDO.setBelongBusinessId(dto.getBelongBusinessId());
            landJobInputsDO.setBelongBusinessName(dto.getBelongBusinessName());
        });
        landJobMapper.insert(landJob);
        landJobInputsMapper.insertBatch(landJobInputsDOS);
        // 返回
        return landJob.getId();
    }

    /**
     * 更新种植/养殖/畜牧作业
     *
     * @param updateReqVO 更新信息
     */
    @Override
    @Transactional
    public void updateLandJob(LandJobUpdateReqVO updateReqVO) {
        // 查询种植计划信息
        LandJobDO landJobById = landJobMapper.selectById(updateReqVO.getId());
        if (landJobById == null) {
            throw exception(LAND_JOB_NOT_EXISTS);
        }

        List<LandJobInputsDO> landJobInputsDOS = LandJobInputsConvert.INSTANCE.convertListCreate(updateReqVO.getLandJobIds());

        // 获取原有的地块作业投入品批次信息
        List<LandJobInputsDO> originalInputsList = landJobInputsMapper.selectList(new QueryWrapper<LandJobInputsDO>()
                .eq("land_job_id", landJobById.getLandJobId()));
        Map<Long, LandJobInputsDO> originalInputsMap = originalInputsList.stream()
                .collect(Collectors.toMap(LandJobInputsDO::getId, Function.identity()));
        // 需要新增的地块作业投入品批次列表
        List<LandJobInputsDO> addList = new ArrayList<>();
        // 需要更新的地块作业投入品批次列表
        List<LandJobInputsDO> updateList = new ArrayList<>();

        for (LandJobInputsDO landJobInputsDO : landJobInputsDOS) {
            Long inputsId = landJobInputsDO.getId();
            LandJobInputsDO originalInputsDO = originalInputsMap.get(inputsId);
            if (Objects.isNull(originalInputsDO)) {
                // 新增流程
                landJobInputsDO.setId(null);
                landJobInputsDO.setBelongBusinessName(landJobById.getBelongBusinessName());
                landJobInputsDO.setBelongBusinessId(landJobById.getBelongBusinessId());
                landJobInputsDO.setLandJobId(landJobById.getLandJobId());
                addList.add(landJobInputsDO);
            } else {// 更新流程

                if (originalInputsDO != null) {
                    // 只更新可更新的字段
                    originalInputsDO.setInputsBatchId(landJobInputsDO.getInputsBatchId());
                    originalInputsDO.setInputsNum(landJobInputsDO.getInputsNum());
                    originalInputsDO.setInputsDesc(landJobInputsDO.getInputsDesc());
                    originalInputsDO.setInputsImgUrl(landJobInputsDO.getInputsImgUrl());
                    originalInputsDO.setInputsVideoUrl(landJobInputsDO.getInputsVideoUrl());
                    updateList.add(originalInputsDO);

                }

            }

            // 批量新增地块作业投入品批次
            if (!addList.isEmpty()) {
                landJobInputsMapper.insertBatch(addList);
            }

            // 批量更新地块作业投入品批次
            if (!updateList.isEmpty()) {
                landJobInputsMapper.updateBatch(updateList, Integer.MAX_VALUE);
            }
        }
        landJobMapper.updateById(LandJobConvert.INSTANCE.convert(updateReqVO));

    }

    /**
     * 删除种植/养殖/畜牧作业
     *
     * @param id 编号
     */
    @Override
    public void deleteLandJob(Long id) {
        // 校验存在
        LandJobDO landJob = landJobMapper.selectById(id);
        if (landJob == null){
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LAND_JOB_NOT_EXISTS);
        }
        // 删除主表数据
        landJobMapper.deleteById(id);
    }

    /**
     * 获得种植/养殖/畜牧作业
     *
     * @param id 编号
     * @return 种植/养殖/畜牧作业
     */
    @Override
    @Transactional
    public LandJobRespVO getLandJob(Long id) {
        LandJobDO landJobDO = landJobMapper.selectById(id);
        if (landJobDO == null){
            throw exception(LAND_JOB_NOT_EXISTS);
        }
        LandJobRespVO landJobRespVO = LandJobConvert.INSTANCE.convert(landJobDO);
        List<LandJobInputsDO> landJobInputsDOList = landJobInputsMapper.selectList(LandJobInputsDO::getLandJobId, landJobRespVO.getLandJobId());
        List<LandJobInputsRespVO> landJobInputsRespVOS = LandJobInputsConvert.INSTANCE.convertList(landJobInputsDOList);
        landJobRespVO.setLandJobIds(landJobInputsRespVOS);
        return landJobRespVO;
    }

    /**
     * 获得种植/养殖/畜牧作业列表
     *
     * @param ids 编号
     * @return 种植/养殖/畜牧作业列表
     */
    @Override
    public List<LandJobDO> getLandJobList(Collection<Long> ids) {
        return landJobMapper.selectBatchIds(ids);
    }

    /**
     * 获得种植/养殖/畜牧作业分页
     *
     * @param pageReqVO 分页查询
     * @return 种植/养殖/畜牧作业分页
     */
    @Override
    public PageResult<LandJobDO> getLandJobPage(LandJobPageReqVO pageReqVO) {
        Page<LandJobDO> page = new Page<>(pageReqVO.getPageNo(),pageReqVO.getPageSize());
        IPage<LandJobDO> landJobDOIPage = landJobMapper.selectJobTypeById(page,pageReqVO.getLandName(),pageReqVO.getLandId(),
                pageReqVO.getJobType(),pageReqVO.getBelongBusinessName());
        PageResult<LandJobDO> landJobDOPageResult = new PageResult<LandJobDO>().setList(landJobDOIPage.getRecords());
        landJobDOPageResult.setTotal(landJobDOIPage.getTotal());
        return landJobDOPageResult;
    }

    /**
     * 获得种植/养殖/畜牧作业列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 种植/养殖/畜牧作业列表
     */
    @Override
    public List<LandJobDO> getLandJobList(LandJobExportReqVO exportReqVO) {
        return landJobMapper.selectList(exportReqVO);
    }

    @Override
    @GlobalTransactional(rollbackFor = Exception.class)
    public void upChain(Long id) {
        LandJobDO landJob = landJobMapper.selectById(id);
        if(landJob == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LAND_JOB_NOT_EXISTS);
        }
        if(!landJob.getBlockState().equals(BlockStateEnum.NOT_ON_CHAIN.getBlockState())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.ONLY_NOT_ON_CHAIN_LAND_JOB_CAN_DO_UP_CHAIN);
        }
        // 封装种植作业
        List<UpChainObjReqDTO> operateJobs = new ArrayList<>();
        UpChainObjReqDTO upChainObjReqDTO = new UpChainObjReqDTO();
        upChainObjReqDTO.setOperateObjId(id);
        operateJobs.add(upChainObjReqDTO);

        BlockCertificateUpChainReqDTO upChainReqDTO = new BlockCertificateUpChainReqDTO();
        upChainReqDTO.setOperateType(OperateTypeEnum.JOB.getType());
        upChainReqDTO.setOperateName(OperateTypeEnum.JOB.getName());
        upChainReqDTO.setOperateObjs(operateJobs);
        upChainReqDTO.setBelongBusinessId(landJob.getBelongBusinessId());
        upChainReqDTO.setBelongBusinessName(landJob.getBelongBusinessName());

        List<BlockCertificateRespDTO> jobResults = blockCertificateApi.createAndUpChainBatch(upChainReqDTO).getData();
        if(jobResults == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.CREATE_BLOCK_CERTIFICATE_FAIL);
        }
        // 将要更新的类
        LandJobDO landJobDO = new LandJobDO();
        landJobDO.setId(id);
        landJobDO.setBlockChainBlockIdentity(jobResults.get(0).getChainBlockIdentity());
        landJobDO.setBlockChainId(jobResults.get(0).getChainOperateId());
        landJobDO.setBlockState(BlockStateEnum.ACTIVATION.getBlockState());// 改为激活状态
        // 更新视频字段
        landJobDO.setJobVideoUrl(landJob.getJobVideoUrl());

        landJobMapper.updateById(landJobDO);
    }

    @Override
    @GlobalTransactional(rollbackFor = Exception.class)
    public void updateChainStatus(Long id, Integer state) {
        LandJobDO LandJob = landJobMapper.selectById(id);
        if(LandJob == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LAND_JOB_NOT_EXISTS);
        }
        if(LandJob.getBlockState().equals(BlockStateEnum.NOT_ON_CHAIN.getBlockState())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.ONLY_ON_CHAIN_LAND_JOB_CAN_DO_UPDATE_CHAIN_STATUS);
        }
        Integer blockState = state.equals(BlockStateEnum.ACTIVATION.getBlockState()) ? BlockStateEnum.ACTIVATION.getBlockState() : BlockStateEnum.FREEZE.getBlockState();
        Integer chainStatus = state.equals(BlockStateEnum.ACTIVATION.getBlockState()) ? ChainStatusEnum.ACTIVE.getStatus() : ChainStatusEnum.FREEZE.getStatus();

        List<UpdateStatusReqDTO> landJobReqDTOS = new ArrayList<>();
        UpdateStatusReqDTO landJobReqDTO = new UpdateStatusReqDTO();
        landJobReqDTO.setChainOperateId(LandJob.getBlockChainId());
        landJobReqDTO.setChainStatus(chainStatus);
        landJobReqDTOS.add(landJobReqDTO);
        if(blockCertificateApi.updateStatusBatch(landJobReqDTOS).isError()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.UPDATE_CREATE_BLOCK_CERTIFICATE_FAIL);
        }

        LandJobDO updateDO = new LandJobDO();
        updateDO.setId(id);
        updateDO.setBlockState(blockState);

        landJobMapper.updateById(updateDO);
    }

    @Override
    public List<LandJobRespVO> getSameTypeJobById(LandJobReqVO landJobReqVO) {
        Long type = null;
        String landId = null;

        if(landJobReqVO.getOperateType().equals(OperateTypeEnum.JOB.getType())) {
            LandJobDO landJobDO = landJobMapper.getLandJob(landJobReqVO.getObjId(), null);
            if(landJobDO != null) {
                type = landJobDO.getJobType();
                landId = landJobDO.getLandId();
            }
        } else {
            LandJobInputsDO landJobInputsDO = landJobInputsMapper.getLandJobInputs(landJobReqVO.getObjId());
            if(landJobInputsDO != null) {
                LandJobDO landJobDO = landJobMapper.getLandJob(null, landJobInputsDO.getLandJobId());
                if(landJobDO != null) {
                    type = landJobDO.getJobType();
                    landId = landJobDO.getLandId();
                }
            }
        }

        if(type != null) {
            return landJobMapper.getSameTypeJobByType(type, landId, landJobReqVO.getDay());
        }
        return new ArrayList<>();
    }

    @Override
    public Long createLandJobByApp(LandJobCreateOfAppReqVO createOfAppReqVO) {
        // 校验商户
        BusinessInfoDTO businessInfo = businessInfoApi.getBusinessInfoByIdWithOutTenantId(createOfAppReqVO.getBelongBusinessId()).getData();
        if(businessInfo == null) {
            throw ServiceExceptionUtil.exception(BUSINESS_INFO_NOT_EXISTS);
        }
        // AtomicReference<Long> atomicId = new AtomicReference<>();

        TenantContextHolder.setTenantId(businessInfo.getTenantId());
        TenantContextHolder.setIgnore(false);
        // 校验地块
        LandInfoDO landInfo = landInfoMapper.selectOne(new LambdaQueryWrapperX<LandInfoDO>().eq(LandInfoDO::getBelongBusinessId, businessInfo.getId())
                .eq(LandInfoDO::getLandId, createOfAppReqVO.getLandId()));
        if(landInfo == null) {
            throw ServiceExceptionUtil.exception(LAND_INFO_NOT_EXISTS);
        }

        // 根据商户ID openid校验员工
        WorkersInfoRespDTO workersInfo = workersinfoApi.getWorkersInfoWithOutTenantId(businessInfo.getId(), createOfAppReqVO.getOpenid()).getData();
        if(workersInfo == null) {
            throw ServiceExceptionUtil.exception(WORKERS_INFO_NOT_EXISTS);
        }

        LandJobCreateReqVO createReqVO = new LandJobCreateReqVO();
        BeanUtil.copyProperties(createOfAppReqVO, createReqVO);
        List<LandJobInputsSaveReqVO> landJobInputs = BeanUtil.copyToList(createOfAppReqVO.getLandJobInputs(), LandJobInputsSaveReqVO.class);
        landJobInputs.forEach(input -> {
            input.setBelongBusinessName(businessInfo.getBusinessName());
        });
        createReqVO.setLandJobIds(landJobInputs);
        createReqVO.setLandType(landInfo.getLandType());
        createReqVO.setLandName(landInfo.getLandName());
        createReqVO.setBelongBusinessName(businessInfo.getBusinessName());
        createReqVO.setJobWorkerId(workersInfo.getId());
        createReqVO.setJobWorker(workersInfo.getWorkersName());

        return this.createLandJob(createReqVO);
    }

    @Override
    public String upload(FileUploadReqVO uploadReqVO) throws IOException {
        // 校验商户
        BusinessInfoDTO businessInfo = businessInfoApi.getBusinessInfoByIdWithOutTenantId(uploadReqVO.getBusinessInfoId()).getData();
        if(businessInfo == null) {
            throw ServiceExceptionUtil.exception(BUSINESS_INFO_NOT_EXISTS);
        }

        // 根据商户ID openid校验员工
        WorkersInfoRespDTO workersInfo = workersinfoApi.getWorkersInfoWithOutTenantId(businessInfo.getId(), uploadReqVO.getOpenid()).getData();
        if(workersInfo == null) {
            throw ServiceExceptionUtil.exception(WORKERS_INFO_NOT_EXISTS);
        }
        AtomicReference<String> url = new AtomicReference<>();
        TenantUtils.execute(businessInfo.getTenantId(), () -> {
            try {
                url.set(fileApi.createFile(uploadReqVO.getFile().getOriginalFilename(), uploadReqVO.getPath(), IoUtil.readBytes(uploadReqVO.getFile().getInputStream())));
            } catch (IOException e) {
                throw ServiceExceptionUtil.exception(FILE_UPLOAD_FAIL);
            }
        });
        return url.get();
    }

    /**
     *
     * @param landJobInputsDOS
     * @param idAndNames 构建投入品编码和投入品昵称映射
     * @param landJobInputsDOMaps 页面传入的LandJobIds集合
     * @param willUpadeteIds 讲要更新的地块作业投入品批次为主键id
     * @param landJobDO 该修改的主库信息
     */
    private void verifInputsExists( List<LandJobInputsDO> landJobInputsDOS, HashMap<String, String> idAndNames,
                                   Map<String, LandJobInputsDO> landJobInputsDOMaps, List<Long> willUpadeteIds, LandJobDO landJobDO ) {

        Set<String> landJobIdsSet = new HashSet<>();
        Set<Long> willUpadeteIdsSet = new HashSet<>();
        landJobInputsDOS.forEach(landJobInputsDO -> {
            landJobIdsSet.add(landJobInputsDO.getInputsId());
            if (landJobInputsDO.getId() != null){
                willUpadeteIdsSet.add(landJobInputsDO.getId());
                if (willUpadeteIds != null){
                    willUpadeteIds.add(landJobInputsDO.getId());
                }
            }
        });
        //存在相同的投入品编码
        if ( landJobIdsSet.size() != landJobInputsDOS.size() ){
            throw exception(INPUTS_INFO_ALREADY_EXIST);
        }
        //存在相同的投入品批次id
        if ( willUpadeteIds != null && willUpadeteIdsSet.size() != willUpadeteIds.size() ){
            throw exception(INPUTS_BATCH_ID_IS_THE_SAME);
        }
        //这边开始校验投入品编码是否存在
        if ( landJobDO != null ){
            List<String> landJobInputsIds = new ArrayList<>(landJobIdsSet);
            List<LandJobInputsDO> landJobInputsDOList = landJobInputsMapper.selectList(new LambdaQueryWrapperX<LandJobInputsDO>()
                    .in(LandJobInputsDO::getInputsId, landJobInputsIds)
                    .eq(LandJobInputsDO::getLandJobId, landJobDO.getLandJobId()));
            landJobInputsDOList.forEach(item ->{
                idAndNames.put(item.getInputsId(),item.getInputsName());
                if ( landJobInputsDOMaps != null ){
                    landJobInputsDOMaps.put(item.getInputsId(),item);
                }
            });
        }
    }

}
