package cn.iocoder.yudao.module.product.service.landjobinputs;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobinputs.LandJobInputsDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.product.convert.landjobinputs.LandJobInputsConvert;
import cn.iocoder.yudao.module.product.dal.mysql.landjobinputs.LandJobInputsMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;

/**
 * 地块作业投入品 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandJobInputsServiceImpl implements LandJobInputsService {

    @Resource
    private LandJobInputsMapper landJobInputsMapper;

    @Override
    public Long createLandJobInputs(LandJobInputsCreateReqVO createReqVO) {
        // 插入
        LandJobInputsDO landJobInputs = LandJobInputsConvert.INSTANCE.convert(createReqVO);
        landJobInputsMapper.insert(landJobInputs);
        // 返回
        return landJobInputs.getId();
    }

    @Override
    public void updateLandJobInputs(LandJobInputsUpdateReqVO updateReqVO) {
        // 校验存在
        validateLandJobInputsExists(updateReqVO.getId());
        // 更新
        LandJobInputsDO updateObj = LandJobInputsConvert.INSTANCE.convert(updateReqVO);
        landJobInputsMapper.updateById(updateObj);
    }

    @Override
    public void deleteLandJobInputs(Long id) {
        LandJobInputsDO landJobInputsDO = landJobInputsMapper.selectById(id);
        if (landJobInputsDO == null){
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LAND_JOB_INPUTS_NOT_EXISTS);
        }
        // 删除
        landJobInputsMapper.deleteById(id);
    }

    private void validateLandJobInputsExists(Long id) {
        if (landJobInputsMapper.selectById(id) == null) {
            throw exception(LAND_JOB_INPUTS_NOT_EXISTS);
        }
    }

    @Override
    public LandJobInputsDO getLandJobInputs(Long id) {
        return landJobInputsMapper.selectById(id);
    }

    @Override
    public List<LandJobInputsDO> getLandJobInputsList(Collection<Long> ids) {
        return landJobInputsMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<LandJobInputsDO> getLandJobInputsPage(LandJobInputsPageReqVO pageReqVO) {
        return landJobInputsMapper.selectPage(pageReqVO);
    }
}
