package cn.iocoder.yudao.module.product.service.landjobtype;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.basic.api.businessinfo.BusinessInfoApi;
import cn.iocoder.yudao.module.basic.api.redis.dto.BusinessInfoDTO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landjobtype.LandJobTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobtype.LandJobTypeDO;
import cn.iocoder.yudao.module.product.dal.mysql.landjobtype.LandJobTypeMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_JOB_TYPE_ALREADY_EXIST;


/**
 * 作业类型 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandJobTypeServiceImpl implements LandJobTypeService {

    @Resource
    private BusinessInfoApi businessInfoApi;

    @Resource
    private LandJobTypeMapper landJobTypeMapper;

    @Override
    public Long createLandJobType(LandJobTypeCreateReqVO createReqVO) {
        //校验商户昵称的作业类型名称是否重复
        validateLandJobTypeNameExists(createReqVO.getTypeName(),createReqVO.getBelongBusinessId());
        // 校验商户
        BusinessInfoDTO businessInfo = businessInfoApi.getBusinessInfoById(createReqVO.getBelongBusinessId()).getData();
        if(businessInfo == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.BUSINESS_INFO_NOT_EXISTS);
        }
        // 插入
        LandJobTypeDO landJobType = LandJobTypeConvert.INSTANCE.convert(createReqVO);
        landJobType.setBelongBusinessName(businessInfo.getBusinessName());
        landJobTypeMapper.insert(landJobType);
        // 返回
        return landJobType.getId();
    }

    @Override
    public void updateLandJobType(LandJobTypeUpdateReqVO updateReqVO) {
        // 校验存在
        LandJobTypeDO selectById = landJobTypeMapper.selectById(updateReqVO.getId());
        if (selectById == null){
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LAND_JOB_TYPE_NOT_EXISTS);
        }
        // 检查地块类型是否发生变化
        if (!Objects.equals(selectById.getTypeName(), updateReqVO.getTypeName())) {
            //校验商户昵称的作业类型名称是否重复
            validateLandJobTypeNameExists(updateReqVO.getTypeName(),updateReqVO.getBelongBusinessId());
        }
        // 更新
        LandJobTypeDO updateObj = LandJobTypeConvert.INSTANCE.convert(updateReqVO);
        landJobTypeMapper.updateById(updateObj);
    }

    @Override
    public void deleteLandJobType(Long id) {
        // 校验存在
        validateLandJobTypeExists(id);
        // 删除
        landJobTypeMapper.deleteById(id);
    }

    private void validateLandJobTypeExists(Long id) {
        if (landJobTypeMapper.selectById(id) == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.LAND_JOB_TYPE_NOT_EXISTS);
        }
    }

    @Override
    public LandJobTypeDO getLandJobType(Long id) {
        return landJobTypeMapper.selectById(id);
    }

    @Override
    public PageResult<LandJobTypeDO> getLandJobTypePage(LandJobTypePageReqVO pageReqVO) {
        return landJobTypeMapper.selectPage(pageReqVO);
    }

    @Override
    public List<LandJobTypeDO> getByBusinessInfoId(Long id) {
        return landJobTypeMapper.selectList(new LambdaQueryWrapperX<LandJobTypeDO>().eq(LandJobTypeDO::getBelongBusinessId, id));
    }

    private void validateLandJobTypeNameExists(String typeName,Long belongBusinessId) {
        QueryWrapper<LandJobTypeDO> wrapper = new QueryWrapper<>();
        wrapper.eq("type_name", typeName)
                .eq("belong_business_id",belongBusinessId);
        if (landJobTypeMapper.selectOne(wrapper) != null) {
            throw exception(LAND_JOB_TYPE_ALREADY_EXIST);
        }
    }
}
