package cn.iocoder.yudao.module.product.service.landplan;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.FarmingPlantReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landplan.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landplan.LandPlanDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 地块作业计划 Service 接口
 *
 * @author 芋道源码
 */
public interface LandPlanService {

    /**
     * 创建地块作业计划
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLandPlan(@Valid LandPlanCreateReqVO createReqVO);

    /**
     * 批量创建地块作业计划
     *
     * @param createReqVOList 创建信息列表
     * @return 编号列表
     */
    List<Long> createLandPlanList(List<LandPlanCreateReqVO> createReqVOList);

    /**
     * 更新地块作业计划
     *
     * @param updateReqVO 更新信息
     */
    void updateLandPlan(@Valid LandPlanUpdateReqVO updateReqVO);

    /**
     * 删除地块作业计划
     *
     * @param id 编号
     */
    void deleteLandPlan(Long id);

    /**
     * 获得地块作业计划
     *
     * @param id 编号
     * @return 地块作业计划
     */
    LandPlanDO getLandPlan(Long id);

    /**
     * 获得地块作业计划列表
     *
     * @param goodsId 编号
     * @return 地块作业计划列表
     */
    List<LandPlanDO> getLandPlanByGoodsId(String goodsId);

    /**
     * 获得地块作业计划分页
     *
     * @param pageReqVO 分页查询
     * @return 地块作业计划分页
     */
    PageResult<LandPlanDO> getLandPlanPage(LandPlanPageReqVO pageReqVO);

    /**
     * 获得地块作业计划列表，用于农事活动配置
     * @param ids 计划id集合
     * @return 农事活动配置
     */
    List<FarmingPlantReqVO> getLandPlanForFarmingConfig(String goodsId, List<Long> ids, List<String> planIds, Integer blockState);

}
