package cn.iocoder.yudao.module.product.service.statistics;

import cn.hutool.core.bean.BeanUtil;
import cn.iocoder.yudao.module.basic.api.goodsinfo.GoodsInfoApi;
import cn.iocoder.yudao.module.basic.api.goodsinfo.dto.GoodsInfoRespDTO;
import cn.iocoder.yudao.module.product.controller.admin.envdata.vo.EnvDataRespVO;
import cn.iocoder.yudao.module.product.controller.admin.statistics.vo.HarvestInfoDataRespVO;
import cn.iocoder.yudao.module.product.controller.admin.statistics.vo.InputsBuyDataRespVO;
import cn.iocoder.yudao.module.product.controller.admin.statistics.vo.TopDataRespVO;
import cn.iocoder.yudao.module.product.dal.dataobject.landjob.LandJobDO;
import cn.iocoder.yudao.module.product.dal.mysql.envdata.EnvDataMapper;
import cn.iocoder.yudao.module.product.dal.mysql.harvestinfo.HarvestInfoMapper;
import cn.iocoder.yudao.module.product.dal.mysql.inputsbuy.InputsBuyMapper;
import cn.iocoder.yudao.module.product.dal.mysql.landinfo.LandInfoMapper;
import cn.iocoder.yudao.module.product.dal.mysql.landjob.LandJobMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.List;

/**
 * 统计数据 service 实现类
 */
@Service
@Validated
public class StatisticsServiceImpl implements StatisticsService {

    @Resource
    private LandInfoMapper landInfoMapper;

    @Resource
    private HarvestInfoMapper harvestInfoMapper;

    @Resource
    private InputsBuyMapper inputsBuyMapper;

    @Resource
    private EnvDataMapper envDataMapper;

    @Resource
    private GoodsInfoApi goodsInfoApi;

    @Resource
    private LandJobMapper landJobMapper;

    @Override
    public TopDataRespVO getTopDataStatistics() {
        TopDataRespVO result = new TopDataRespVO();
        BeanUtil.copyProperties(landInfoMapper.getLandStatistics(), result);

        List<GoodsInfoRespDTO> goodsInfoList = goodsInfoApi.getAllGoodsList().getData();
        if (goodsInfoList != null) {
            result.setGoodsTypeCount(goodsInfoList.size());
        } else {
            result.setGoodsTypeCount(BigInteger.ZERO.intValue());
        }

        // 查询产量
        BigDecimal nums = harvestInfoMapper.getNumStatisticsOfYear();
        if (nums != null) {
            result.setTotalHarvestCount(nums);
        } else {
            result.setTotalHarvestCount(BigDecimal.ZERO);
        }
        return result;
    }

    @Override
    public HarvestInfoDataRespVO getHarvestInfoDataStatistics() {
        HarvestInfoDataRespVO result = new HarvestInfoDataRespVO();
        // 查询月同比
        result.setMonthYOYDatas(harvestInfoMapper.getMonthYOYDatas());

        // 查询周同比
        result.setWeekYOYDatas(harvestInfoMapper.getWeekYOYDatas());

        // 查询本月采收，按照品种分组
        result.setGoodsMonthStatisticsDatas(harvestInfoMapper.getDatasGroupByGoodsId("month"));

        // 查询本周采收，按照品种分组
        result.setGoodsWeekStatisticsDatas(harvestInfoMapper.getDatasGroupByGoodsId("week"));

        // 查询今日采收数量
        result.setTodayHarvestCount(harvestInfoMapper.getTodayHarvestCount());
        return result;
    }

    @Override
    public InputsBuyDataRespVO getInputsBuyDataStatistics() {
        InputsBuyDataRespVO result = new InputsBuyDataRespVO();
        // 查询月同比
        result.setMonthYOYDatas(inputsBuyMapper.getMonthYOYDatas());

        // 查询周同比
        result.setWeekYOYDatas(inputsBuyMapper.getWeekYOYDatas());

        // 查询本月采收，按照品种分组
        result.setInputsMonthStatisticsDatas(inputsBuyMapper.getDatasGroupByGoodsId("month"));

        // 查询本周采收，按照品种分组
        result.setInputsWeekStatisticsDatas(inputsBuyMapper.getDatasGroupByGoodsId("week"));

        // 查询今日采收数量
        result.setTodayInputBuyCount(inputsBuyMapper.getTodayInputBuyCount());
        return result;
    }

    /**
     * 获取首页农事数据记录
     *
     * @param
     * @return 返回10条数据
     */
    @Override
    public List<LandJobDO> getLandJobListStatistics() {
        return landJobMapper.selectLandJob10Records();
    }

    @Override
    public List<EnvDataRespVO> getAvgEnvDataStatistics() {
        return envDataMapper.getAVGEnvDatas(null, 7, null);

    }
}
