package cn.iocoder.yudao.module.product.service.inputsbatch;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbatch.InputsBatchDO;
import cn.iocoder.yudao.module.product.dal.mysql.inputsbatch.InputsBatchMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import org.springframework.context.annotation.Import;
import java.util.*;

import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static org.junit.jupiter.api.Assertions.*;

/**
* {@link InputsBatchServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(InputsBatchServiceImpl.class)
public class InputsBatchServiceImplTest extends BaseDbUnitTest {

    @Resource
    private InputsBatchServiceImpl inputsBatchService;

    @Resource
    private InputsBatchMapper inputsBatchMapper;

    @Test
    public void testCreateInputsBatch_success() {
        // 准备参数
        InputsBatchCreateReqVO reqVO = randomPojo(InputsBatchCreateReqVO.class);

        // 调用
        Long inputsBatchId = inputsBatchService.createInputsBatch(reqVO);
        // 断言
        assertNotNull(inputsBatchId);
        // 校验记录的属性是否正确
        InputsBatchDO inputsBatch = inputsBatchMapper.selectById(inputsBatchId);
        assertPojoEquals(reqVO, inputsBatch);
    }

    @Test
    public void testUpdateInputsBatch_success() {
        // mock 数据
        InputsBatchDO dbInputsBatch = randomPojo(InputsBatchDO.class);
        inputsBatchMapper.insert(dbInputsBatch);// @Sql: 先插入出一条存在的数据
        // 准备参数
        InputsBatchSaveReqVO reqVO = randomPojo(InputsBatchSaveReqVO.class, o -> {
            o.setId(dbInputsBatch.getId()); // 设置更新的 ID
        });

        // 调用
        inputsBatchService.updateInputsBatch(reqVO);
        // 校验是否更新正确
        InputsBatchDO inputsBatch = inputsBatchMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, inputsBatch);
    }

    @Test
    public void testUpdateInputsBatch_notExists() {
        // 准备参数
        InputsBatchSaveReqVO reqVO = randomPojo(InputsBatchSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> inputsBatchService.updateInputsBatch(reqVO), INPUTS_BATCH_NOT_EXISTS);
    }

    @Test
    public void testDeleteInputsBatch_success() {
        // mock 数据
        InputsBatchDO dbInputsBatch = randomPojo(InputsBatchDO.class);
        inputsBatchMapper.insert(dbInputsBatch);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbInputsBatch.getId();

        // 调用
        inputsBatchService.deleteInputsBatch(id);
       // 校验数据不存在了
       assertNull(inputsBatchMapper.selectById(id));
    }

    @Test
    public void testDeleteInputsBatch_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> inputsBatchService.deleteInputsBatch(id), INPUTS_BATCH_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetInputsBatchPage() {
       // mock 数据
       InputsBatchDO dbInputsBatch = randomPojo(InputsBatchDO.class, o -> { // 等会查询到
           o.setInputsName(null);
           o.setInputsId(null);
           o.setInputsBatchId(null);
           o.setInputsBuyId(null);
           o.setProductData(null);
           o.setInputsNum(null);
           o.setInputsUnit(null);
           o.setInputsWeight(null);
           o.setInputsUnitPrice(null);
           o.setTotalPrice(null);
           o.setRemarks(null);
           o.setBlockState(null);
           o.setBlockChainId(null);
           o.setBlockChainBlockIdentity(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       inputsBatchMapper.insert(dbInputsBatch);
       // 测试 inputsName 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsName(null)));
       // 测试 inputsId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsId(null)));
       // 测试 inputsBatchId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsBatchId(null)));
       // 测试 inputsBuyId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsBuyId(null)));
       // 测试 productData 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setProductData(null)));
       // 测试 inputsNum 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsNum(null)));
       // 测试 inputsUnit 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsUnit(null)));
       // 测试 inputsWeight 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsWeight(null)));
       // 测试 inputsUnitPrice 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsUnitPrice(null)));
       // 测试 totalPrice 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setTotalPrice(null)));
       // 测试 remarks 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setRemarks(null)));
       // 测试 blockState 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBlockState(null)));
       // 测试 blockChainId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBlockChainId(null)));
       // 测试 blockChainBlockIdentity 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBlockChainBlockIdentity(null)));
       // 测试 belongBusinessId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setCreateTime(null)));
       // 准备参数
       InputsBatchPageReqVO reqVO = new InputsBatchPageReqVO();
       reqVO.setInputsName(null);
       reqVO.setInputsBatchId(null);
       reqVO.setInputsBuyId(null);
       reqVO.setProductData(null);
       reqVO.setBlockState(null);
       reqVO.setBelongBusinessName(null);
       // reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<InputsBatchDO> pageResult = inputsBatchService.getInputsBatchPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbInputsBatch, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetInputsBatchList() {
       // mock 数据
       InputsBatchDO dbInputsBatch = randomPojo(InputsBatchDO.class, o -> { // 等会查询到
           o.setInputsName(null);
           o.setInputsId(null);
           o.setInputsBatchId(null);
           o.setInputsBuyId(null);
           o.setProductData(null);
           o.setInputsNum(null);
           o.setInputsUnit(null);
           o.setInputsWeight(null);
           o.setInputsUnitPrice(null);
           o.setTotalPrice(null);
           o.setRemarks(null);
           o.setBlockState(null);
           o.setBlockChainId(null);
           o.setBlockChainBlockIdentity(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       inputsBatchMapper.insert(dbInputsBatch);
       // 测试 inputsName 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsName(null)));
       // 测试 inputsId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsId(null)));
       // 测试 inputsBatchId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsBatchId(null)));
       // 测试 inputsBuyId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsBuyId(null)));
       // 测试 productData 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setProductData(null)));
       // 测试 inputsNum 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsNum(null)));
       // 测试 inputsUnit 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsUnit(null)));
       // 测试 inputsWeight 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsWeight(null)));
       // 测试 inputsUnitPrice 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setInputsUnitPrice(null)));
       // 测试 totalPrice 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setTotalPrice(null)));
       // 测试 remarks 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setRemarks(null)));
       // 测试 blockState 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBlockState(null)));
       // 测试 blockChainId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBlockChainId(null)));
       // 测试 blockChainBlockIdentity 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBlockChainBlockIdentity(null)));
       // 测试 belongBusinessId 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       inputsBatchMapper.insert(cloneIgnoreId(dbInputsBatch, o -> o.setCreateTime(null)));
       // 准备参数
       InputsBatchExportReqVO reqVO = new InputsBatchExportReqVO();
       reqVO.setInputsName(null);
       reqVO.setInputsId(null);
       reqVO.setInputsBatchId(null);
       reqVO.setInputsBuyId(null);
       reqVO.setProductData(null);
       reqVO.setInputsNum(null);
       reqVO.setInputsUnit(null);
       reqVO.setInputsWeight(null);
       reqVO.setInputsUnitPrice(null);
       reqVO.setTotalPrice(null);
       reqVO.setRemarks(null);
       reqVO.setBlockState(null);
       reqVO.setBlockChainId(null);
       reqVO.setBlockChainBlockIdentity(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
       // reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       List<InputsBatchDO> list = inputsBatchService.getInputsBatchList(reqVO);
       // 断言
       assertEquals(1, list.size());
       assertPojoEquals(dbInputsBatch, list.get(0));
    }

}
