package cn.iocoder.yudao.module.product.service.inputsbuy;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.LocalDateTimeUtils;
import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyPageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyRespVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbuy.vo.InputsBuyUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbuy.InputsBuyDO;
import cn.iocoder.yudao.module.product.dal.mysql.inputsbuy.InputsBuyMapper;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.cloneIgnoreId;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.INPUTS_BUY_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.*;

/**
* {@link InputsBuyServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(InputsBuyServiceImpl.class)
public class InputsBuyServiceImplTest extends BaseDbUnitTest {

    @Resource
    private InputsBuyServiceImpl inputsBuyService;

    @Resource
    private InputsBuyMapper inputsBuyMapper;

    @Test
    public void testCreateInputsBuy_success() {
        // 准备参数
        InputsBuyCreateReqVO reqVO = randomPojo(InputsBuyCreateReqVO.class);

        // 调用
        Long inputsBuyId = inputsBuyService.createInputsBuy(reqVO);
        // 断言
        assertNotNull(inputsBuyId);
        // 校验记录的属性是否正确
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(inputsBuyId);
        assertPojoEquals(reqVO, inputsBuy);
    }

    @Test
    public void testUpdateInputsBuy_success() {
        // mock 数据
        InputsBuyDO dbInputsBuy = randomPojo(InputsBuyDO.class);
        inputsBuyMapper.insert(dbInputsBuy);// @Sql: 先插入出一条存在的数据
        // 准备参数
        InputsBuyUpdateReqVO reqVO = randomPojo(InputsBuyUpdateReqVO.class, o -> {
            o.setId(dbInputsBuy.getId()); // 设置更新的 ID
        });

        // 调用
        inputsBuyService.updateInputsBuy(reqVO);
        // 校验是否更新正确
        InputsBuyDO inputsBuy = inputsBuyMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, inputsBuy);
    }

    @Test
    public void testUpdateInputsBuy_notExists() {
        // 准备参数
        InputsBuyUpdateReqVO reqVO = randomPojo(InputsBuyUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> inputsBuyService.updateInputsBuy(reqVO), INPUTS_BUY_NOT_EXISTS);
    }

    @Test
    public void testDeleteInputsBuy_success() {
        // mock 数据
        InputsBuyDO dbInputsBuy = randomPojo(InputsBuyDO.class);
        inputsBuyMapper.insert(dbInputsBuy);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbInputsBuy.getId();

        // 调用
        inputsBuyService.deleteInputsBuy(id);
       // 校验数据不存在了
       assertNull(inputsBuyMapper.selectById(id));
    }

    @Test
    public void testDeleteInputsBuy_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> inputsBuyService.deleteInputsBuy(id), INPUTS_BUY_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetInputsBuyPage() {
       // mock 数据
       InputsBuyDO dbInputsBuy = randomPojo(InputsBuyDO.class, o -> { // 等会查询到
           o.setInputsBuyId(null);
           o.setRelateOutboundId(null);
           o.setVendorName(null);
           o.setVendorId(null);
           o.setInputsBuyTime(null);
           o.setCreateTime(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
       });
       inputsBuyMapper.insert(dbInputsBuy);
       // 测试 inputsBuyId 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setInputsBuyId(null)));
       // 测试 relateOutboundId 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setRelateOutboundId(null)));
       // 测试 vendorName 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setVendorName(null)));
       // 测试 vendorId 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setVendorId(null)));
       // 测试 inputsBuyTime 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setInputsBuyTime(null)));
       // 测试 createTime 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setCreateTime(null)));
       // 测试 belongBusinessId 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       inputsBuyMapper.insert(cloneIgnoreId(dbInputsBuy, o -> o.setBelongBusinessName(null)));
       // 准备参数
       InputsBuyPageReqVO reqVO = new InputsBuyPageReqVO();
       reqVO.setInputsBuyId(null);
       reqVO.setRelateOutboundId(null);
       reqVO.setVendorName(null);
       reqVO.setVendorId(null);

       LocalDateTime[] localDateTimes = LocalDateTimeUtils.buildBetweenTime(2023, 2, 1, 2023, 2, 28);
       Date[] dates = new Date[localDateTimes.length];
       for(int i=0; i<localDateTimes.length; i++) {
           dates[i] = Date.from(localDateTimes[i].atZone(ZoneId.systemDefault()).toInstant());
       }

       reqVO.setInputsBuyTime(dates);
       reqVO.setCreateTime(dates);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);

       // 调用
       PageResult<InputsBuyRespVO> pageResult = inputsBuyService.getInputsBuyPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbInputsBuy, pageResult.getList().get(0));
    }

}
