package cn.iocoder.yudao.module.product.service.landinfo;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfo.LandInfoDO;
import cn.iocoder.yudao.module.product.dal.mysql.landinfo.LandInfoMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
* {@link LandInfoServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(LandInfoServiceImpl.class)
public class LandInfoServiceImplTest extends BaseDbUnitTest {

    @Resource
    private LandInfoServiceImpl landInfoService;

    @Resource
    private LandInfoMapper landInfoMapper;

    @Test
    public void testCreateLandInfo_success() {
        // 准备参数
        LandInfoCreateReqVO reqVO = randomPojo(LandInfoCreateReqVO.class);

        // 调用
        Long landInfoId = landInfoService.createLandInfo(reqVO);
        // 断言
        assertNotNull(landInfoId);
        // 校验记录的属性是否正确
        LandInfoDO landInfo = landInfoMapper.selectById(landInfoId);
        assertPojoEquals(reqVO, landInfo);
    }

    @Test
    public void testUpdateLandInfo_success() {
        // mock 数据
        LandInfoDO dbLandInfo = randomPojo(LandInfoDO.class);
        landInfoMapper.insert(dbLandInfo);// @Sql: 先插入出一条存在的数据
        // 准备参数
        LandInfoUpdateReqVO reqVO = randomPojo(LandInfoUpdateReqVO.class, o -> {
            o.setId(dbLandInfo.getId()); // 设置更新的 ID
        });

        // 调用
        landInfoService.updateLandInfo(reqVO);
        // 校验是否更新正确
        LandInfoDO landInfo = landInfoMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, landInfo);
    }

    @Test
    public void testUpdateLandInfo_notExists() {
        // 准备参数
        LandInfoUpdateReqVO reqVO = randomPojo(LandInfoUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> landInfoService.updateLandInfo(reqVO), LAND_INFO_NOT_EXISTS);
    }

    @Test
    public void testDeleteLandInfo_success() {
        // mock 数据
        LandInfoDO dbLandInfo = randomPojo(LandInfoDO.class);
        landInfoMapper.insert(dbLandInfo);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbLandInfo.getId();

        // 调用
        landInfoService.deleteLandInfo(id);
       // 校验数据不存在了
       assertNull(landInfoMapper.selectById(id));
    }

    @Test
    public void testDeleteLandInfo_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> landInfoService.deleteLandInfo(id), LAND_INFO_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetLandInfoPage() {
       // mock 数据
       LandInfoDO dbLandInfo = randomPojo(LandInfoDO.class, o -> { // 等会查询到
           o.setLandName(null);
           o.setLandId(null);
           o.setSize(null);
           o.setState(null);
           o.setRemarks(null);
           o.setLandType(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       landInfoMapper.insert(dbLandInfo);
       // 测试 landName 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setLandName(null)));
       // 测试 landId 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setLandId(null)));
       // 测试 size 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setSize(null)));
       // 测试 state 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setState(null)));
       // 测试 remarks 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setRemarks(null)));
       // 测试 landType 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setLandType(null)));
       // 测试 belongBusinessId 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       landInfoMapper.insert(cloneIgnoreId(dbLandInfo, o -> o.setCreateTime(null)));
       // 准备参数
       LandInfoPageReqVO reqVO = new LandInfoPageReqVO();
       reqVO.setLandName(null);
       reqVO.setLandId(null);
       reqVO.setSize(null);
       reqVO.setState(null);
       reqVO.setRemarks(null);
       reqVO.setLandType(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<LandInfoDO> pageResult = landInfoService.getLandInfoPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbLandInfo, pageResult.getList().get(0));
    }
}
