package cn.iocoder.yudao.module.product.service.landjob;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landjob.LandJobDO;
import cn.iocoder.yudao.module.product.dal.mysql.landjob.LandJobMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
* {@link LandJobServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(LandJobServiceImpl.class)
public class LandJobServiceImplTest extends BaseDbUnitTest {

    @Resource
    private LandJobServiceImpl landJobService;

    @Resource
    private LandJobMapper landJobMapper;

    @Test
    public void testCreateLandJob_success() {
        // 准备参数
        LandJobCreateReqVO reqVO = randomPojo(LandJobCreateReqVO.class);

        // 调用
        Long landJobId = landJobService.createLandJob(reqVO);
        // 断言
        assertNotNull(landJobId);
        // 校验记录的属性是否正确
        LandJobDO landJob = landJobMapper.selectById(landJobId);
        assertPojoEquals(reqVO, landJob);
    }

    @Test
    public void testUpdateLandJob_success() {
        // mock 数据
        LandJobDO dbLandJob = randomPojo(LandJobDO.class);
        landJobMapper.insert(dbLandJob);// @Sql: 先插入出一条存在的数据
        // 准备参数
        LandJobUpdateReqVO reqVO = randomPojo(LandJobUpdateReqVO.class, o -> {
            o.setId(dbLandJob.getId()); // 设置更新的 ID
        });

        // 调用
        landJobService.updateLandJob(reqVO);
        // 校验是否更新正确
        LandJobDO landJob = landJobMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, landJob);
    }

    @Test
    public void testUpdateLandJob_notExists() {
        // 准备参数
        LandJobUpdateReqVO reqVO = randomPojo(LandJobUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> landJobService.updateLandJob(reqVO), LAND_JOB_NOT_EXISTS);
    }

    @Test
    public void testDeleteLandJob_success() {
        // mock 数据
        LandJobDO dbLandJob = randomPojo(LandJobDO.class);
        landJobMapper.insert(dbLandJob);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbLandJob.getId();

        // 调用
        landJobService.deleteLandJob(id);
       // 校验数据不存在了
       assertNull(landJobMapper.selectById(id));
    }

    @Test
    public void testDeleteLandJob_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> landJobService.deleteLandJob(id), LAND_JOB_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetLandJobPage() {
       // mock 数据
       LandJobDO dbLandJob = randomPojo(LandJobDO.class, o -> { // 等会查询到
           o.setLandId(null);
           o.setLandName(null);
           o.setLandType(null);
           o.setJobType(null);
           o.setJobWorker(null);
           o.setJobWorkerId(null);
           o.setJobDate(null);
           o.setRemarks(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
           o.setBlockState(null);
       });
       landJobMapper.insert(dbLandJob);
       // 测试 landId 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setLandId(null)));
       // 测试 landName 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setLandName(null)));
       // 测试 landType 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setLandType(null)));
       // 测试 jobType 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobType(null)));
       // 测试 jobWorker 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobWorker(null)));
       // 测试 jobWorkerId 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobWorkerId(null)));
       // 测试 jobDate 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobDate(null)));
       // 测试 remarks 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setRemarks(null)));
       // 测试 belongBusinessId 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setCreateTime(null)));
       // 测试 blockState 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setBlockState(null)));
       // 准备参数
       LandJobPageReqVO reqVO = new LandJobPageReqVO();
       reqVO.setLandId(null);
       reqVO.setLandName(null);
       reqVO.setLandType(null);
       reqVO.setJobType(null);
       reqVO.setJobWorker(null);
       reqVO.setJobWorkerId(null);
//       reqVO.setJobDate(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setRemarks(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setBlockState(null);

       // 调用
       PageResult<LandJobDO> pageResult = landJobService.getLandJobPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbLandJob, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetLandJobList() {
       // mock 数据
       LandJobDO dbLandJob = randomPojo(LandJobDO.class, o -> { // 等会查询到
           o.setLandId(null);
           o.setLandName(null);
           o.setLandType(null);
           o.setJobType(null);
           o.setJobWorker(null);
           o.setJobWorkerId(null);
           o.setJobDate(null);
           o.setRemarks(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
           o.setBlockState(null);
       });
       landJobMapper.insert(dbLandJob);
       // 测试 landId 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setLandId(null)));
       // 测试 landName 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setLandName(null)));
       // 测试 landType 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setLandType(null)));
       // 测试 jobType 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobType(null)));
       // 测试 jobWorker 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobWorker(null)));
       // 测试 jobWorkerId 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobWorkerId(null)));
       // 测试 jobDate 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setJobDate(null)));
       // 测试 remarks 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setRemarks(null)));
       // 测试 belongBusinessId 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setCreateTime(null)));
       // 测试 blockState 不匹配
       landJobMapper.insert(cloneIgnoreId(dbLandJob, o -> o.setBlockState(null)));
       // 准备参数
       LandJobExportReqVO reqVO = new LandJobExportReqVO();
       reqVO.setLandId(null);
       reqVO.setLandName(null);
       reqVO.setLandType(null);
       reqVO.setJobType(null);
       reqVO.setJobWorker(null);
       reqVO.setJobWorkerId(null);
//       reqVO.setJobDate(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setRemarks(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setBlockState(null);

       // 调用
       List<LandJobDO> list = landJobService.getLandJobList(reqVO);
       // 断言
       assertEquals(1, list.size());
       assertPojoEquals(dbLandJob, list.get(0));
    }

}
