package cn.iocoder.yudao.module.product.service.landjobinputs;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobinputs.LandJobInputsDO;
import cn.iocoder.yudao.module.product.dal.mysql.landjobinputs.LandJobInputsMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
* {@link LandJobInputsServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(LandJobInputsServiceImpl.class)
public class LandJobInputsServiceImplTest extends BaseDbUnitTest {

    @Resource
    private LandJobInputsServiceImpl landJobInputsService;

    @Resource
    private LandJobInputsMapper landJobInputsMapper;

    @Test
    public void testCreateLandJobInputs_success() {
        // 准备参数
        LandJobInputsCreateReqVO reqVO = randomPojo(LandJobInputsCreateReqVO.class);

        // 调用
        Long landJobInputsId = landJobInputsService.createLandJobInputs(reqVO);
        // 断言
        assertNotNull(landJobInputsId);
        // 校验记录的属性是否正确
        LandJobInputsDO landJobInputs = landJobInputsMapper.selectById(landJobInputsId);
        assertPojoEquals(reqVO, landJobInputs);
    }

    @Test
    public void testUpdateLandJobInputs_success() {
        // mock 数据
        LandJobInputsDO dbLandJobInputs = randomPojo(LandJobInputsDO.class);
        landJobInputsMapper.insert(dbLandJobInputs);// @Sql: 先插入出一条存在的数据
        // 准备参数
        LandJobInputsUpdateReqVO reqVO = randomPojo(LandJobInputsUpdateReqVO.class, o -> {
            o.setId(dbLandJobInputs.getId()); // 设置更新的 ID
        });

        // 调用
        landJobInputsService.updateLandJobInputs(reqVO);
        // 校验是否更新正确
        LandJobInputsDO landJobInputs = landJobInputsMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, landJobInputs);
    }

    @Test
    public void testUpdateLandJobInputs_notExists() {
        // 准备参数
        LandJobInputsUpdateReqVO reqVO = randomPojo(LandJobInputsUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> landJobInputsService.updateLandJobInputs(reqVO), LAND_JOB_INPUTS_NOT_EXISTS);
    }

    @Test
    public void testDeleteLandJobInputs_success() {
        // mock 数据
        LandJobInputsDO dbLandJobInputs = randomPojo(LandJobInputsDO.class);
        landJobInputsMapper.insert(dbLandJobInputs);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbLandJobInputs.getId();

        // 调用
        landJobInputsService.deleteLandJobInputs(id);
       // 校验数据不存在了
       assertNull(landJobInputsMapper.selectById(id));
    }

    @Test
    public void testDeleteLandJobInputs_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> landJobInputsService.deleteLandJobInputs(id), LAND_JOB_INPUTS_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetLandJobInputsPage() {
       // mock 数据
       LandJobInputsDO dbLandJobInputs = randomPojo(LandJobInputsDO.class, o -> { // 等会查询到
           o.setLandJobId(null);
           o.setInputsName(null);
           o.setInputsId(null);
           o.setInputsBatchId(null);
           o.setInputsNum(null);
           o.setInputsUnit(null);
           o.setInputsDesc(null);
           o.setCreateTime(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
       });
       landJobInputsMapper.insert(dbLandJobInputs);
       // 测试 landJobId 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setLandJobId(null)));
       // 测试 inputsName 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setInputsName(null)));
       // 测试 inputsId 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setInputsId(null)));
       // 测试 inputsBatchId 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setInputsBatchId(null)));
       // 测试 inputsNum 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setInputsNum(null)));
       // 测试 inputsUnit 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setInputsUnit(null)));
       // 测试 inputsDesc 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setInputsDesc(null)));
       // 测试 createTime 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setCreateTime(null)));
       // 测试 belongBusinessId 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       landJobInputsMapper.insert(cloneIgnoreId(dbLandJobInputs, o -> o.setBelongBusinessName(null)));
       // 准备参数
       LandJobInputsPageReqVO reqVO = new LandJobInputsPageReqVO();
       reqVO.setLandJobId(null);
       reqVO.setInputsName(null);
       reqVO.setInputsId(null);
       reqVO.setInputsBatchId(null);
       reqVO.setInputsNum(null);
       reqVO.setInputsUnit(null);
       reqVO.setInputsDesc(null);
       reqVO.setCreateTime(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);

       // 调用
       PageResult<LandJobInputsDO> pageResult = landJobInputsService.getLandJobInputsPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbLandJobInputs, pageResult.getList().get(0));
    }
}
