package cn.iocoder.yudao.module.product.service.landjobtype;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobtype.vo.LandJobTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.landjobtype.LandJobTypeDO;
import cn.iocoder.yudao.module.product.dal.mysql.landjobtype.LandJobTypeMapper;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_JOB_TYPE_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

/**
* {@link LandJobTypeServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(LandJobTypeServiceImpl.class)
public class LandJobTypeServiceImplTest extends BaseDbUnitTest {

    @Resource
    private LandJobTypeServiceImpl landJobTypeService;

    @Resource
    private LandJobTypeMapper landJobTypeMapper;

    @Test
    public void testCreateLandJobType_success() {
        // 准备参数
        LandJobTypeCreateReqVO reqVO = randomPojo(LandJobTypeCreateReqVO.class);

        // 调用
        Long landJobTypeId = landJobTypeService.createLandJobType(reqVO);
        // 断言
        assertNotNull(landJobTypeId);
        // 校验记录的属性是否正确
        LandJobTypeDO landJobType = landJobTypeMapper.selectById(landJobTypeId);
        assertPojoEquals(reqVO, landJobType);
    }

    @Test
    public void testUpdateLandJobType_success() {
        // mock 数据
        LandJobTypeDO dbLandJobType = randomPojo(LandJobTypeDO.class);
        landJobTypeMapper.insert(dbLandJobType);// @Sql: 先插入出一条存在的数据
        // 准备参数
        LandJobTypeUpdateReqVO reqVO = randomPojo(LandJobTypeUpdateReqVO.class, o -> {
            o.setId(dbLandJobType.getId()); // 设置更新的 ID
        });

        // 调用
        landJobTypeService.updateLandJobType(reqVO);
        // 校验是否更新正确
        LandJobTypeDO landJobType = landJobTypeMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, landJobType);
    }

    @Test
    public void testUpdateLandJobType_notExists() {
        // 准备参数
        LandJobTypeUpdateReqVO reqVO = randomPojo(LandJobTypeUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> landJobTypeService.updateLandJobType(reqVO), LAND_JOB_TYPE_NOT_EXISTS);
    }

    @Test
    public void testDeleteLandJobType_success() {
        // mock 数据
        LandJobTypeDO dbLandJobType = randomPojo(LandJobTypeDO.class);
        landJobTypeMapper.insert(dbLandJobType);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbLandJobType.getId();

        // 调用
        landJobTypeService.deleteLandJobType(id);
       // 校验数据不存在了
       assertNull(landJobTypeMapper.selectById(id));
    }

    @Test
    public void testDeleteLandJobType_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> landJobTypeService.deleteLandJobType(id), LAND_JOB_TYPE_NOT_EXISTS);
    }

}
