package cn.iocoder.yudao.module.product.service.landplan;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.product.controller.admin.landplan.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.landplan.LandPlanDO;
import cn.iocoder.yudao.module.product.dal.mysql.landplan.LandPlanMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
* {@link LandPlanServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(LandPlanServiceImpl.class)
public class LandPlanServiceImplTest extends BaseDbUnitTest {

    @Resource
    private LandPlanServiceImpl landPlanService;

    @Resource
    private LandPlanMapper landPlanMapper;

    @Test
    public void testCreateLandPlan_success() {
        // 准备参数
        LandPlanCreateReqVO reqVO = randomPojo(LandPlanCreateReqVO.class);

        // 调用
        Long landPlanId = landPlanService.createLandPlan(reqVO);
        // 断言
        assertNotNull(landPlanId);
        // 校验记录的属性是否正确
        LandPlanDO landPlan = landPlanMapper.selectById(landPlanId);
        assertPojoEquals(reqVO, landPlan);
    }

    @Test
    public void testUpdateLandPlan_success() {
        // mock 数据
        LandPlanDO dbLandPlan = randomPojo(LandPlanDO.class);
        landPlanMapper.insert(dbLandPlan);// @Sql: 先插入出一条存在的数据
        // 准备参数
        LandPlanUpdateReqVO reqVO = randomPojo(LandPlanUpdateReqVO.class, o -> {
            o.setId(dbLandPlan.getId()); // 设置更新的 ID
        });

        // 调用
        landPlanService.updateLandPlan(reqVO);
        // 校验是否更新正确
        LandPlanDO landPlan = landPlanMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, landPlan);
    }

    @Test
    public void testUpdateLandPlan_notExists() {
        // 准备参数
        LandPlanUpdateReqVO reqVO = randomPojo(LandPlanUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> landPlanService.updateLandPlan(reqVO), LAND_PLAN_NOT_EXISTS);
    }

    @Test
    public void testDeleteLandPlan_success() {
        // mock 数据
        LandPlanDO dbLandPlan = randomPojo(LandPlanDO.class);
        landPlanMapper.insert(dbLandPlan);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbLandPlan.getId();

        // 调用
        landPlanService.deleteLandPlan(id);
       // 校验数据不存在了
       assertNull(landPlanMapper.selectById(id));
    }

    @Test
    public void testDeleteLandPlan_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> landPlanService.deleteLandPlan(id), LAND_PLAN_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetLandPlanPage() {
       // mock 数据
       LandPlanDO dbLandPlan = randomPojo(LandPlanDO.class, o -> { // 等会查询到
           o.setLandId(null);
           o.setLandName(null);
           o.setLandPlanId(null);
           o.setGoodsId(null);
           o.setGoodsName(null);
           o.setPlanStartTime(null);
           o.setPlanEndTime(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setRemark(null);
           o.setCreateTime(null);
       });
       landPlanMapper.insert(dbLandPlan);
       // 测试 landId 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setLandId(null)));
       // 测试 landName 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setLandName(null)));
       // 测试 landPlanId 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setLandPlanId(null)));
       // 测试 goodsId 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setGoodsId(null)));
       // 测试 goodName 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setGoodsName(null)));
       // 测试 planStartTime 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setPlanStartTime(null)));
       // 测试 planEndTime 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setPlanEndTime(null)));
       // 测试 belongBusinessId 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setBelongBusinessName(null)));
       // 测试 remark 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setRemark(null)));
       // 测试 createTime 不匹配
       landPlanMapper.insert(cloneIgnoreId(dbLandPlan, o -> o.setCreateTime(null)));
       // 准备参数
       LandPlanPageReqVO reqVO = new LandPlanPageReqVO();
       reqVO.setLandId(null);
       reqVO.setLandName(null);
       reqVO.setLandPlanId(null);
       reqVO.setGoodsId(null);
       reqVO.setGoodsName(null);
//       reqVO.setPlanStartTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
//       reqVO.setPlanEndTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
       reqVO.setRemark(null);
//       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<LandPlanDO> pageResult = landPlanService.getLandPlanPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbLandPlan, pageResult.getList().get(0));
    }
}
